@@ -25,9 +25,17 @@
  * rr uses this fd to ensure the tracee has access to the original root
  * directory after a chroot(). Tracee close()es of this fd will be silently
  * ignored, and tracee dup()s to this fd will fail with EBADF.
+ * This is set up during both recording and replay.
  */
 #define RR_RESERVED_ROOT_DIR_FD 1000
 
+/**
+ * Tracees use this fd to send other fds to rr.
+ * This is only set up during recording.
+ * Only the outermost rr uses this. Inner rr replays will use a different fd.
+ */
+#define RR_RESERVED_SOCKET_FD 1001
+
 /**
  * The preferred fd that rr uses to control tracee desched. Some software
  * (e.g. the chromium IPC code) wants to have the first few fds all to itself,
