@@ -177,7 +177,7 @@ KernelMapping AddressSpace::read_local_kernel_mapping(uint8_t* addr) {
 /**
  * Cat the /proc/[t->tid]/maps file to stdout, line by line.
  */
-static void print_process_mmap(Task* t) {
+void AddressSpace::print_process_maps(Task* t) {
   for (KernelMapIterator it(t); !it.at_end(); ++it) {
     string line;
     it.current(&line);
@@ -258,7 +258,7 @@ remote_code_ptr AddressSpace::find_syscall_instruction(Task* t) {
 }
 
 static string find_rr_page_file(Task* t) {
-  string path = exe_directory() + "../bin/rr_page_";
+  string path = resource_path() + "bin/rr_page_";
   switch (t->arch()) {
     case x86:
       path += "32";
@@ -315,6 +315,8 @@ void AddressSpace::map_rr_page(AutoRemoteSyscalls& remote) {
     map(t, rr_page_start(), rr_page_size(), prot, flags, 0, file_name,
         fstat.st_dev, fstat.st_ino);
   } else {
+    ASSERT(t, child_fd != -ENOENT) << "rr_page file not found: "
+        << path.c_str();
     ASSERT(t, child_fd == -EACCES) << "Unexpected error mapping rr_page";
     flags |= MAP_ANONYMOUS;
     remote.infallible_mmap_syscall(rr_page_start(), rr_page_size(), prot, flags,
@@ -425,42 +427,6 @@ vector<AddressSpace::SyscallType> AddressSpace::rr_page_syscalls() {
   return result;
 }
 
-template <typename Arch> static vector<uint8_t> read_auxv_arch(Task* t) {
-  auto stack_ptr = t->regs().sp().cast<typename Arch::unsigned_word>();
-
-  auto argc = t->read_mem(stack_ptr);
-  stack_ptr += argc + 1;
-
-  // Check final NULL in argv
-  auto null_ptr = t->read_mem(stack_ptr);
-  ASSERT(t, null_ptr == 0);
-  stack_ptr++;
-
-  // Should now point to envp
-  while (0 != t->read_mem(stack_ptr)) {
-    stack_ptr++;
-  }
-  stack_ptr++;
-  // should now point to ELF Auxiliary Table
-
-  vector<uint8_t> result;
-  while (true) {
-    auto pair_vec = t->read_mem(stack_ptr, 2);
-    stack_ptr += 2;
-    typename Arch::unsigned_word pair[2] = { pair_vec[0], pair_vec[1] };
-    result.resize(result.size() + sizeof(pair));
-    memcpy(result.data() + result.size() - sizeof(pair), pair, sizeof(pair));
-    if (pair[0] == 0) {
-      break;
-    }
-  }
-  return result;
-}
-
-static vector<uint8_t> read_auxv(Task* t) {
-  RR_ARCH_FUNCTION(read_auxv_arch, t->arch(), t);
-}
-
 void AddressSpace::save_auxv(Task* t) { saved_auxv_ = read_auxv(t); }
 
 void AddressSpace::post_exec_syscall(Task* t) {
@@ -507,8 +473,6 @@ static const char* stringify_flags(int flags) {
       return " [thread_locals]";
     case AddressSpace::Mapping::IS_PATCH_STUBS:
       return " [patch_stubs]";
-    case AddressSpace::Mapping::IS_SIGBUS_REGION:
-      return " [sigbus_region]";
     default:
       return "[unknown_flags]";
   }
@@ -832,12 +796,12 @@ void AddressSpace::remap(Task* t, remote_ptr<void> old_addr,
                          size_t new_num_bytes) {
   LOG(debug) << "mremap(" << old_addr << ", " << old_num_bytes << ", "
              << new_addr << ", " << new_num_bytes << ")";
+  old_num_bytes = ceil_page_size(old_num_bytes);
 
   Mapping mr = mapping_of(old_addr);
   DEBUG_ASSERT(!mr.monitored_shared_memory);
-  const KernelMapping& m = mr.map;
+  KernelMapping km = mr.map.subrange(old_addr, min(mr.map.end(), old_addr + old_num_bytes));
 
-  old_num_bytes = ceil_page_size(old_num_bytes);
   unmap_internal(t, old_addr, old_num_bytes);
   if (0 == new_num_bytes) {
     return;
@@ -855,8 +819,10 @@ void AddressSpace::remap(Task* t, remote_ptr<void> old_addr,
     remove_range(dont_fork, MemoryRange(new_addr, new_num_bytes));
   }
 
+  unmap_internal(t, new_addr, new_num_bytes);
+
   remote_ptr<void> new_end = new_addr + new_num_bytes;
-  map_and_coalesce(t, m.set_range(new_addr, new_end),
+  map_and_coalesce(t, km.set_range(new_addr, new_end),
                    mr.recorded_map.set_range(new_addr, new_end), mr.emu_file,
                    clone_stat(mr.mapped_file_stat), nullptr, nullptr);
 }
@@ -1299,12 +1265,66 @@ static void assert_segments_match(Task* t, const KernelMapping& input_m,
     LOG(error) << "cached mmap:";
     t->vm()->dump();
     LOG(error) << "/proc/" << t->tid << "/mmaps:";
-    print_process_mmap(t);
+    AddressSpace::print_process_maps(t);
     ASSERT(t, false) << "\nCached mapping " << m << " should be " << km << "; "
                      << err;
   }
 }
 
+void AddressSpace::ensure_replay_matches_single_recorded_mapping(Task* t, MemoryRange range) {
+  // The only case where we eagerly coalesced during recording but not replay should
+  // be where we mapped private memory beyond-end-of-file.
+  // Don't do an actual coalescing check here; we rely on the caller to tell us
+  // the range to coalesce.
+  ASSERT(t, range.start() == floor_page_size(range.start()));
+  ASSERT(t, range.end() == ceil_page_size(range.end()));
+
+  auto fixer = [this, t, range](const Mapping& mm, const MemoryRange&) {
+    if (mm.map == range) {
+      // Existing single mapping covers entire range; nothing to do.
+      return;
+    }
+    Mapping mapping = move(mm);
+
+    // These should be null during replay
+    ASSERT(t, !mapping.mapped_file_stat);
+    // These should not be in use for a beyond-end-of-file mapping
+    ASSERT(t, !mapping.local_addr);
+    // The mapping should be private
+    ASSERT(t, mapping.map.flags() & MAP_PRIVATE);
+    ASSERT(t, !mapping.emu_file);
+    ASSERT(t, !mapping.monitored_shared_memory);
+    // Flagged mappings shouldn't be coalescable ever
+    ASSERT(t, !mapping.flags);
+
+    if (!(mapping.map.flags() & MAP_ANONYMOUS)) {
+      // Direct-mapped piece. Turn it into an anonymous mapping.
+      vector<uint8_t> buffer;
+      buffer.resize(mapping.map.size());
+      t->read_bytes_helper(mapping.map.start(), buffer.size(), buffer.data());
+      {
+        AutoRemoteSyscalls remote(t);
+        remote.infallible_mmap_syscall(mapping.map.start(), buffer.size(),
+            mapping.map.prot(), mapping.map.flags() | MAP_ANONYMOUS | MAP_FIXED, -1, 0);
+      }
+      t->write_bytes_helper(mapping.map.start(), buffer.size(), buffer.data());
+
+      // We replace the entire mapping even if part of it falls outside the desired range.
+      // That's OK, this replacement preserves behaviour, it's simpler, even if a bit
+      // less efficient in weird cases.
+      mem.erase(mapping.map);
+      KernelMapping anonymous_km(mapping.map.start(), mapping.map.end(),
+                                 string(), KernelMapping::NO_DEVICE, KernelMapping::NO_INODE,
+                                 mapping.map.prot(), mapping.map.flags() | MAP_ANONYMOUS);
+      Mapping new_mapping(anonymous_km, mapping.recorded_map);
+      mem[new_mapping.map] = new_mapping;
+    }
+  };
+  for_each_in_range(range.start(), range.size(), fixer);
+
+  coalesce_around(t, mem.find(range));
+}
+
 KernelMapping AddressSpace::vdso() const {
   DEBUG_ASSERT(!vdso_start_addr.is_null());
   return mapping_of(vdso_start_addr).map;
@@ -1433,13 +1453,13 @@ AddressSpace::AddressSpace(Session* session, const AddressSpace& o,
   // cloned address-space memory, so we don't need to do any more work here.
 }
 
-void AddressSpace::post_vm_clone(Task* t) {
+bool AddressSpace::post_vm_clone(Task* t) {
   if (has_mapping(preload_thread_locals_start()) &&
       (mapping_flags_of(preload_thread_locals_start()) &
        AddressSpace::Mapping::IS_THREAD_LOCALS) == 0) {
     // The tracee already has a mapping at this address that doesn't belong to
     // us. Don't touch it.
-    return;
+    return false;
   }
 
   // Otherwise, the preload_thread_locals mapping is non-existent or ours.
@@ -1450,6 +1470,7 @@ void AddressSpace::post_vm_clone(Task* t) {
                                   "preload_thread_locals");
   mapping_flags_of(preload_thread_locals_start()) |=
       AddressSpace::Mapping::IS_THREAD_LOCALS;
+  return true;
 }
 
 static bool try_split_unaligned_range(MemoryRange& range, size_t bytes,
@@ -1864,10 +1885,10 @@ static MemoryRange choose_global_exclusion_range() {
   uint64_t r_addr = r & ((uint64_t(1) << bits) - 1);
   r_addr = min(r_addr, (uint64_t(1) << bits) - range_size);
   remote_ptr<void> addr = floor_page_size(remote_ptr<void>(r_addr));
-  return MemoryRange(addr, range_size);
+  return MemoryRange(addr, (uintptr_t)range_size);
 }
 
-remote_ptr<void> AddressSpace::chaos_mode_find_free_memory(Task* t,
+remote_ptr<void> AddressSpace::chaos_mode_find_free_memory(RecordTask* t,
                                                            size_t len) {
   static MemoryRange global_exclusion_range = choose_global_exclusion_range();
 
@@ -1922,11 +1943,22 @@ remote_ptr<void> AddressSpace::chaos_mode_find_free_memory(Task* t,
       // We fell off one end of the address space. Try everything again.
       continue;
     }
-
     MemoryRange r(addr, ceil_page_size(len));
-    if (!r.intersects(global_exclusion_range)) {
-      return addr;
+    if (r.intersects(global_exclusion_range)) {
+      continue;
     }
+    if (t->session().asan_active() && sizeof(size_t) == 8) {
+      LOG(debug) << "Checking ASAN shadow";
+      MemoryRange asan_shadow(remote_ptr<void>((uintptr_t)0x00007fff7000LL),
+                              remote_ptr<void>((uintptr_t)0x10007fff8000LL));
+      MemoryRange asan_allocator_reserved(remote_ptr<void>((uintptr_t)0x600000000000LL),
+                                          remote_ptr<void>((uintptr_t)0x640000000000LL));
+      if (r.intersects(asan_shadow) || r.intersects(asan_allocator_reserved)) {
+        continue;
+      }
+    }
+
+    return addr;
   }
 }
 
