@@ -21,8 +21,9 @@ void CPUIDBugDetector::run_detection_code() {
   cpuid_loop(4);
 }
 
-static bool rcb_counts_ok(uint64_t prev, uint64_t current) {
-  if (current - prev == 2) {
+static bool rcb_counts_ok(ReplayTask* t, uint64_t prev, uint64_t current) {
+  uint32_t expected_count = 2 + PerfCounters::ticks_for_direct_call(t);
+  if (current - prev == expected_count) {
     return true;
   }
   if (!Flags::get().suppress_environment_warnings) {
@@ -56,8 +57,8 @@ void CPUIDBugDetector::notify_reached_syscall_during_replay(ReplayTask* t) {
   uint64_t trace_rcb_count = t->current_trace_frame().ticks();
   uint64_t actual_rcb_count = t->tick_count();
   if (trace_rcb_count_at_last_geteuid32 > 0 && !detected_cpuid_bug) {
-    if (!rcb_counts_ok(trace_rcb_count_at_last_geteuid32, trace_rcb_count) ||
-        !rcb_counts_ok(actual_rcb_count_at_last_geteuid32, actual_rcb_count)) {
+    if (!rcb_counts_ok(t, trace_rcb_count_at_last_geteuid32, trace_rcb_count) ||
+        !rcb_counts_ok(t, actual_rcb_count_at_last_geteuid32, actual_rcb_count)) {
       detected_cpuid_bug = true;
     }
   }
