@@ -1,5 +1,7 @@
 /* -*- Mode: C++; tab-width: 8; c-basic-offset: 2; indent-tabs-mode: nil; -*- */
 
+#include "DumpCommand.h"
+
 #include <inttypes.h>
 
 #include <limits>
@@ -45,25 +47,6 @@ DumpCommand DumpCommand::singleton(
     "  -s, --statistics           dump statistics about the trace\n"
     "  -t, --tid=<pid>            dump events only for the specified tid\n");
 
-struct DumpFlags {
-  bool dump_syscallbuf;
-  bool dump_recorded_data_metadata;
-  bool dump_mmaps;
-  bool dump_task_events;
-  bool raw_dump;
-  bool dump_statistics;
-  int only_tid;
-
-  DumpFlags()
-      : dump_syscallbuf(false),
-        dump_recorded_data_metadata(false),
-        dump_mmaps(false),
-        dump_task_events(false),
-        raw_dump(false),
-        dump_statistics(false),
-        only_tid(0) {}
-};
-
 static bool parse_dump_arg(vector<string>& args, DumpFlags& flags) {
   if (parse_global_option(args)) {
     return true;
@@ -285,6 +268,9 @@ static void dump_events_matching(TraceReader& trace, const DumpFlags& flags,
           break;
         }
       }
+      TraceReader::RawDataMetadata data;
+      while (process_raw_data && trace.read_raw_data_metadata_for_frame(data)) {
+      }
     }
   }
 }
@@ -297,8 +283,8 @@ static void dump_statistics(const TraceReader& trace, FILE* out) {
           uncompressed, compressed, double(uncompressed) / compressed);
 }
 
-static void dump(const string& trace_dir, const DumpFlags& flags,
-                 const vector<string>& specs, FILE* out) {
+void dump(const string& trace_dir, const DumpFlags& flags,
+          const vector<string>& specs, FILE* out) {
   TraceReader trace(trace_dir);
 
   if (flags.raw_dump) {
@@ -309,15 +295,15 @@ static void dump(const string& trace_dir, const DumpFlags& flags,
 
   if (specs.size() > 0) {
     for (size_t i = 0; i < specs.size(); ++i) {
-      dump_events_matching(trace, flags, stdout, &specs[i]);
+      dump_events_matching(trace, flags, out, &specs[i]);
     }
   } else {
     // No specs => dump all events.
-    dump_events_matching(trace, flags, stdout, nullptr /*all events*/);
+    dump_events_matching(trace, flags, out, nullptr /*all events*/);
   }
 
   if (flags.dump_statistics) {
-    dump_statistics(trace, stdout);
+    dump_statistics(trace, out);
   }
 }
 
