@@ -21,6 +21,7 @@
 #include <sys/types.h>
 #include <unistd.h>
 
+#include <iomanip>
 #include <sstream>
 #include <vector>
 
@@ -359,14 +360,21 @@ static void read_binary_data(const uint8_t* payload, const uint8_t* payload_end,
 static GdbThreadId parse_threadid(const char* str, char** endptr) {
   GdbThreadId t;
   char* endp;
+  bool multiprocess = false;
 
   if ('p' == *str) {
+    multiprocess = true;
     ++str;
   }
   t.pid = strtol(str, &endp, 16);
   parser_assert(endp);
   if ('\0' == *endp) {
-    t.tid = -1;
+    if (multiprocess) {
+      t.tid = -1;
+    } else {
+      t.tid = t.pid;
+      t.pid = -1;
+    }
     *endptr = endp;
     return t;
   }
@@ -398,7 +406,7 @@ void GdbConnection::write_xfer_response(const void* data, size_t size,
 }
 
 static string read_target_desc(const char* file_name) {
-  string path = exe_directory() + "../share/rr/" + string(file_name);
+  string path = resource_path() + "share/rr/" + string(file_name);
   stringstream ss;
   FILE* f = fopen(path.c_str(), "r");
   DEBUG_ASSERT(f);
@@ -643,6 +651,8 @@ bool GdbConnection::query(char* payload) {
     /* TODO process these */
     LOG(debug) << "gdb supports " << args;
 
+    multiprocess_supported_ = strstr(args, "multiprocess+") != nullptr;
+
     stringstream supported;
     // Encourage gdb to use very large packets since we support any packet size
     supported << "PacketSize=1048576"
@@ -653,7 +663,8 @@ bool GdbConnection::query(char* payload) {
                  ";qXfer:siginfo:read+"
                  ";qXfer:siginfo:write+"
                  ";multiprocess+"
-                 ";ConditionalBreakpoints+";
+                 ";ConditionalBreakpoints+"
+                 ";vContSupported+";
     if (features().reverse_execution) {
       supported << ";ReverseContinue+"
                    ";ReverseStep+";
@@ -1035,10 +1046,10 @@ static string to_string(const vector<uint8_t>& bytes, size_t max_len) {
 }
 
 bool GdbConnection::process_packet() {
-  parser_assert(INTERRUPT_CHAR == inbuf[0] ||
-                ('$' == inbuf[0] &&
-                 (uint8_t*)memchr(inbuf.data(), '#', inbuf.size()) ==
-                     inbuf.data() + packetend));
+  parser_assert(
+      INTERRUPT_CHAR == inbuf[0] ||
+      ('$' == inbuf[0] && (uint8_t*)memchr(inbuf.data(), '#', inbuf.size()) ==
+                              inbuf.data() + packetend));
 
   if (INTERRUPT_CHAR == inbuf[0]) {
     LOG(debug) << "gdb requests interrupt";
@@ -1057,6 +1068,13 @@ bool GdbConnection::process_packet() {
     case 'b':
       ret = process_bpacket(payload);
       break;
+    case 'c':
+      LOG(debug) << "gdb is asking to continue";
+      req = GdbRequest(DREQ_CONT);
+      req.cont().run_direction = RUN_FORWARD;
+      req.cont().actions.push_back(GdbContAction(ACTION_CONTINUE));
+      ret = true;
+      break;
     case 'D':
       LOG(debug) << "gdb is detaching from us";
       req = GdbRequest(DREQ_DETACH);
@@ -1438,8 +1456,13 @@ void GdbConnection::send_stop_reply_packet(GdbThreadId thread, int sig,
     watch[0] = '\0';
   }
   char buf[PATH_MAX];
-  snprintf(buf, sizeof(buf) - 1, "T%02xthread:p%02x.%02x;%s",
+  if (multiprocess_supported_) {
+    snprintf(buf, sizeof(buf) - 1, "T%02xthread:p%02x.%02x;%s",
            to_gdb_signum(sig), thread.pid, thread.tid, watch);
+  } else {
+    snprintf(buf, sizeof(buf) - 1, "T%02xthread:%02x;%s",
+           to_gdb_signum(sig), thread.tid, watch);
+  }
   write_packet(buf);
 }
 
@@ -1488,7 +1511,11 @@ void GdbConnection::reply_get_current_thread(GdbThreadId thread) {
   DEBUG_ASSERT(DREQ_GET_CURRENT_THREAD == req.type);
 
   char buf[1024];
-  snprintf(buf, sizeof(buf), "QCp%02x.%02x", thread.pid, thread.tid);
+  if (multiprocess_supported_) {
+    snprintf(buf, sizeof(buf), "QCp%02x.%02x", thread.pid, thread.tid);
+  } else {
+    snprintf(buf, sizeof(buf), "QC%02x", thread.tid);
+  }
   write_packet(buf);
 
   consume_request();
@@ -1607,7 +1634,8 @@ void GdbConnection::reply_get_reg(const GdbRegisterValue& reg) {
 }
 
 void GdbConnection::reply_get_regs(const vector<GdbRegisterValue>& file) {
-  char buf[file.size() * 2 * GdbRegisterValue::MAX_SIZE + 1];
+  std::unique_ptr<char[]> buf(
+      new char[file.size() * 2 * GdbRegisterValue::MAX_SIZE + 1]);
 
   DEBUG_ASSERT(DREQ_GET_REGS == req.type);
 
@@ -1615,7 +1643,7 @@ void GdbConnection::reply_get_regs(const vector<GdbRegisterValue>& file) {
   for (auto& reg : file) {
     offset += print_reg_value(reg, &buf[offset]);
   }
-  write_packet(buf);
+  write_packet(buf.get());
 
   consume_request();
 }
@@ -1646,31 +1674,28 @@ void GdbConnection::reply_get_stop_reason(GdbThreadId which, int sig) {
 
 void GdbConnection::reply_get_thread_list(const vector<GdbThreadId>& threads) {
   DEBUG_ASSERT(DREQ_GET_THREAD_LIST == req.type);
-
   if (threads.empty()) {
     write_packet("l");
   } else {
-    ssize_t maxlen =
-        1 /*m char*/ +
-        threads.size() * (1 /*p*/ + 2 * sizeof(threads[0]) + 1 /*,*/) +
-        1 /*\0*/;
-    char* str = (char*)xmalloc(maxlen);
-    int offset = 0;
-
-    str[offset++] = 'm';
+    stringstream sstr;
+    sstr << 'm';
     for (size_t i = 0; i < threads.size(); ++i) {
       const GdbThreadId& t = threads[i];
       if (tgid != t.pid) {
         continue;
       }
-      offset +=
-          snprintf(&str[offset], maxlen - offset, "p%02x.%02x,", t.pid, t.tid);
+      if (multiprocess_supported_) {
+        sstr << 'p' << setw(2) << setfill('0') << hex << t.pid << dec << '.'
+            << setw(2) << setfill('0') << hex << t.tid << ',';
+      } else {
+        sstr << setw(2) << setfill('0') << hex << t.tid << ',';
+      }
     }
-    /* Overwrite the trailing ',' */
-    str[offset - 1] = '\0';
 
-    write_packet(str);
-    free(str);
+    string str = sstr.str();
+    /* Overwrite the trailing ',' */
+    str.back() = 0;
+    write_packet(str.c_str());
   }
 
   consume_request();
