@@ -53,9 +53,14 @@ static const string& gdb_rr_macros() {
 
   if (s.empty()) {
     stringstream ss;
-    ss << "define restart\n"
+    ss << GdbCommandHandler::gdb_macros()
+       << "define restart\n"
        << "  run c$arg0\n"
        << "end\n"
+       << "document restart\n"
+       << "restart at checkpoint N\n"
+       << "checkpoints are created with the 'checkpoint' command\n"
+       << "end\n"
        // In gdb version "Fedora 7.8.1-30.fc21", a raw "run" command
        // issued before any user-generated resume-execution command
        // results in gdb hanging just after the inferior hits an internal
@@ -67,47 +72,44 @@ static const string& gdb_rr_macros() {
        // issue the "stepi" command, then gdb refuses to restart
        // execution.
        << "define hook-run\n"
-       << "  if $_thread != 0 && !$suppress_run_hook\n"
-       << "    stepi\n"
-       << "  end\n"
+       << "  rr-hook-run\n"
        << "end\n"
        << "define hookpost-continue\n"
-       << "  set $suppress_run_hook = 1\n"
+       << "  rr-set-suppress-run-hook 1\n"
        << "end\n"
        << "define hookpost-step\n"
-       << "  set $suppress_run_hook = 1\n"
+       << "  rr-set-suppress-run-hook 1\n"
        << "end\n"
        << "define hookpost-stepi\n"
-       << "  set $suppress_run_hook = 1\n"
+       << "  rr-set-suppress-run-hook 1\n"
        << "end\n"
        << "define hookpost-next\n"
-       << "  set $suppress_run_hook = 1\n"
+       << "  rr-set-suppress-run-hook 1\n"
        << "end\n"
        << "define hookpost-nexti\n"
-       << "  set $suppress_run_hook = 1\n"
+       << "  rr-set-suppress-run-hook 1\n"
        << "end\n"
        << "define hookpost-finish\n"
-       << "  set $suppress_run_hook = 1\n"
+       << "  rr-set-suppress-run-hook 1\n"
        << "end\n"
        << "define hookpost-reverse-continue\n"
-       << "  set $suppress_run_hook = 1\n"
+       << "  rr-set-suppress-run-hook 1\n"
        << "end\n"
        << "define hookpost-reverse-step\n"
-       << "  set $suppress_run_hook = 1\n"
+       << "  rr-set-suppress-run-hook 1\n"
        << "end\n"
        << "define hookpost-reverse-stepi\n"
-       << "  set $suppress_run_hook = 1\n"
+       << "  rr-set-suppress-run-hook 1\n"
        << "end\n"
        << "define hookpost-reverse-finish\n"
-       << "  set $suppress_run_hook = 1\n"
+       << "  rr-set-suppress-run-hook 1\n"
        << "end\n"
        << "define hookpost-run\n"
-       << "  set $suppress_run_hook = 0\n"
+       << "  rr-set-suppress-run-hook 0\n"
        << "end\n"
        << "set unwindonsignal on\n"
        << "handle SIGURG stop\n"
        << "set prompt (rr) \n"
-       << GdbCommandHandler::gdb_macros()
        // Try both "set target-async" and "maint set target-async" since
        // that changed recently.
        << "python\n"
@@ -310,14 +312,6 @@ static bool search_memory(Task* t, const MemoryRange& where,
   return false;
 }
 
-template <typename Arch> static size_t word_size_arch() {
-  return sizeof(typename Arch::signed_long);
-}
-
-static size_t word_size(SupportedArch arch) {
-  RR_ARCH_FUNCTION(word_size_arch, arch);
-}
-
 static bool is_in_patch_stubs(Task* t, remote_code_ptr ip) {
   auto p = ip.to_data_ptr<void>();
   return t->vm()->has_mapping(p) &&
