@@ -204,6 +204,7 @@ static remote_ptr<uint8_t> allocate_extended_jump(
                    &recorded);
       t->vm()->mapping_flags_of(addr) |= AddressSpace::Mapping::IS_PATCH_STUBS;
       t->trace_writer().write_mapped_region(t, recorded, recorded.fake_stat(),
+                                            vector<TraceRemoteFd>(),
                                             TraceWriter::PATCH_MAPPING);
     }
 
@@ -288,11 +289,12 @@ static bool patch_syscall_with_hook_x86ish(Monkeypatcher& patcher,
   static const uint8_t NOP = 0x90;
   DEBUG_ASSERT(syscall_instruction_length(x86_64) ==
                syscall_instruction_length(x86));
-  uint8_t nops[syscall_instruction_length(x86_64) +
-               hook.next_instruction_length - sizeof(jump_patch)];
-  memset(nops, NOP, sizeof(nops));
-  write_and_record_mem(t, jump_patch_start + sizeof(jump_patch), nops,
-                       sizeof(nops));
+  size_t nops_bufsize = syscall_instruction_length(x86_64) +
+                        hook.next_instruction_length - sizeof(jump_patch);
+  std::unique_ptr<uint8_t[]> nops(new uint8_t[nops_bufsize]);
+  memset(nops.get(), NOP, nops_bufsize);
+  write_and_record_mem(t, jump_patch_start + sizeof(jump_patch), nops.get(),
+                       nops_bufsize);
 
   return true;
 }
@@ -486,13 +488,14 @@ bool Monkeypatcher::try_patch_syscall(RecordTask* t) {
 
 class VdsoReader : public ElfReader {
 public:
-  VdsoReader(RecordTask* t) : ElfReader(t->arch()), t(t) {}
-  virtual bool read(size_t offset, size_t size, void* buf) override {
-    bool ok = true;
-    t->read_bytes_helper(t->vm()->vdso().start() + offset, size, buf, &ok);
-    return ok;
+  VdsoReader(RecordTask* t) : ElfReader(t->arch()) {
+    size = t->vm()->vdso().size();
+    map = new uint8_t[size];
+    t->read_bytes_helper(t->vm()->vdso().start(), size, map);
+  }
+  virtual ~VdsoReader() {
+    delete[] map;
   }
-  RecordTask* t;
 };
 
 /**
@@ -573,22 +576,22 @@ struct named_syscall {
   int syscall_number;
 };
 
-static void erase_section(VdsoReader& reader, const char* name) {
+static void erase_section(RecordTask* t, VdsoReader& reader, const char* name) {
   SectionOffsets offsets = reader.find_section_file_offsets(name);
   if (offsets.end > offsets.start) {
     vector<uint8_t> zeroes;
     zeroes.resize(offsets.end - offsets.start);
     memset(zeroes.data(), 0, zeroes.size());
-    write_and_record_bytes(reader.t,
-        reader.t->vm()->vdso().start() + offsets.start,
+    write_and_record_bytes(t,
+        t->vm()->vdso().start() + offsets.start,
         offsets.end - offsets.start, zeroes.data());
   }
 }
 
-static void obliterate_debug_info(VdsoReader& reader) {
-  erase_section(reader, ".eh_frame");
-  erase_section(reader, ".eh_frame_hdr");
-  erase_section(reader, ".note");
+static void obliterate_debug_info(RecordTask* t, VdsoReader& reader) {
+  erase_section(t, reader, ".eh_frame");
+  erase_section(t, reader, ".eh_frame_hdr");
+  erase_section(t, reader, ".note");
 }
 
 // Monkeypatch x86-32 vdso syscalls immediately after exec. The vdso syscalls
@@ -662,7 +665,7 @@ void patch_after_exec_arch<X86Arch>(RecordTask* t, Monkeypatcher& patcher) {
       }
     }
   }
-  obliterate_debug_info(reader);
+  obliterate_debug_info(t, reader);
 }
 
 // Monkeypatch x86 vsyscall hook only after the preload library
@@ -729,11 +732,7 @@ void patch_after_exec_arch<X64Arch>(RecordTask* t, Monkeypatcher& patcher) {
 
   static const named_syscall syscalls_to_monkeypatch[] = {
 #define S(n) { "__vdso_" #n, X64Arch::n }
-    S(clock_gettime), S(gettimeofday), S(time),
-    // getcpu isn't supported by rr, so any changes to this monkeypatching
-    // scheme for efficiency's sake will have to ensure that getcpu gets
-    // converted to an actual syscall so rr will complain appropriately.
-    S(getcpu),
+    S(clock_gettime), S(gettimeofday), S(time), S(getcpu),
 #undef S
   };
 
@@ -787,7 +786,7 @@ void patch_after_exec_arch<X64Arch>(RecordTask* t, Monkeypatcher& patcher) {
     }
   }
 
-  obliterate_debug_info(reader);
+  obliterate_debug_info(t, reader);
 
   for (const auto& m : t->vm()->maps()) {
     auto& km = m.map;
