@@ -20,6 +20,11 @@ namespace rr {
 
 class Task;
 
+enum TicksSemantics {
+  TICKS_RETIRED_CONDITIONAL_BRANCHES,
+  TICKS_TAKEN_BRANCHES,
+};
+
 /**
  * A class encapsulating the performance counters we use to monitor
  * each task during recording and replay.
@@ -37,7 +42,7 @@ public:
   /**
    * Create performance counters monitoring the given task.
    */
-  PerfCounters(pid_t tid);
+  PerfCounters(pid_t tid, TicksSemantics ticks_semantics);
   ~PerfCounters() { stop(); }
 
   void set_tid(pid_t tid);
@@ -66,12 +71,26 @@ public:
    */
   void stop_counting();
 
+  /**
+   * Return the number of ticks we need for an emulated branch.
+   */
+  static Ticks ticks_for_unconditional_indirect_branch(Task*);
+  /**
+   * Return the number of ticks we need for a direct call.
+   */
+  static Ticks ticks_for_direct_call(Task*);
+
   /**
    * Read the current value of the ticks counter.
    * `t` is used for debugging purposes.
    */
   Ticks read_ticks(Task* t);
 
+  /**
+   * Returns what ticks mean for these counters.
+   */
+  TicksSemantics ticks_semantics() const { return ticks_semantics_; }
+
   /**
    * Return the fd we last used to generate the ticks-counter signal.
    */
@@ -81,7 +100,11 @@ public:
    * hope that tracees don't either. */
   enum { TIME_SLICE_SIGNAL = SIGSTKFLT };
 
-  static bool is_ticks_attr(const perf_event_attr& attr);
+  static bool is_rr_ticks_attr(const perf_event_attr& attr);
+
+  static bool supports_ticks_semantics(TicksSemantics ticks_semantics);
+
+  static TicksSemantics default_ticks_semantics();
 
   /**
    * When an interrupt is requested, at most this many ticks may elapse before
@@ -105,9 +128,11 @@ private:
   // sample_period; the latter does not ignore ticks in aborted transactions,
   // but does support sample_period.
   ScopedFd fd_ticks_measure;
+  ScopedFd fd_minus_ticks_measure;
   ScopedFd fd_ticks_interrupt;
   ScopedFd fd_ticks_in_transaction;
   ScopedFd fd_useless_counter;
+  TicksSemantics ticks_semantics_;
   bool started;
   bool counting;
 };
