@@ -3,7 +3,9 @@
 #include "RecordCommand.h"
 
 #include <linux/capability.h>
+#include <spawn.h>
 #include <sys/prctl.h>
+#include <sys/wait.h>
 #include <sysexits.h>
 
 #include "preload/preload_interface.h"
@@ -13,6 +15,7 @@
 #include "StringVectorToCharArray.h"
 #include "WaitStatus.h"
 #include "core.h"
+#include "git_revision.h"
 #include "kernel_metadata.h"
 #include "log.h"
 #include "main.h"
@@ -53,10 +56,20 @@ RecordCommand RecordCommand::singleton(
     "  --no-file-cloning          disable file cloning for mmapped files\n"
     "  --no-read-cloning          disable file-block cloning for syscallbuf\n"
     "                             reads\n"
+    "  --num-cores=N              pretend to have N cores (rr will still\n"
+    "                             only run on a single core). Overrides\n"
+    "                             random setting from --chaos.\n"
+    "  -o, --output-trace-dir<DIR> set the output trace directory.\n"
+    "                             _RR_TRACE_DIR gets ignored.\n"
+    "                             Directory name is given name, not the\n"
+    "                             application name.\n"
     "  -p --print-trace-dir=<NUM> print trace directory followed by a newline\n"
     "                             to given file descriptor\n"
     "  --syscall-buffer-size=<NUM> desired size of syscall buffer in kB.\n"
     "                             Mainly for tests\n"
+    "  --syscall-buffer-sig=<NUM> the signal used for communication with the\n"
+    "                             syscall buffer. SIGPWR by default, unused\n"
+    "                             if --no-syscall-buffer is passed\n"
     "  -s, --always-switch        try to context switch at every rr event\n"
     "  -t, --continue-through-signal=<SIG>\n"
     "                             Unhandled <SIG> signals will be ignored\n"
@@ -81,7 +94,9 @@ RecordCommand RecordCommand::singleton(
     "                             (for testing purposes)\n"
     "  --setuid-sudo              If running under sudo, pretend to be the\n"
     "                             user that ran sudo rather than root. This\n"
-    "                             allows recording setuid/setcap binaries.\n");
+    "                             allows recording setuid/setcap binaries.\n"
+    "  --trace-id                 Sets the trace id to the specified id.\n"
+    "  --copy-preload-src         Copy preload sources to trace dir\n");
 
 struct RecordFlags {
   vector<string> extra_env;
@@ -109,6 +124,8 @@ struct RecordFlags {
 
   int print_trace_dir;
 
+  string output_trace_dir;
+
   /* Whether to use file-cloning optimization during recording. */
   bool use_file_cloning;
 
@@ -125,6 +142,9 @@ struct RecordFlags {
   /* Whether to enable chaos mode in the scheduler */
   bool chaos;
 
+  /* Controls number of cores reported to recorded process. */
+  int num_cores;
+
   /* True if we should wait for all processes to exit before finishing
    * recording. */
   bool wait_for_all;
@@ -136,6 +156,14 @@ struct RecordFlags {
 
   bool setuid_sudo;
 
+  unique_ptr<TraceUuid> trace_id;
+
+  /* Copy preload sources to trace dir */
+  bool copy_preload_src;
+
+  /* The signal to use for syscallbuf desched events */
+  int syscallbuf_desched_sig;
+
   RecordFlags()
       : max_ticks(Scheduler::DEFAULT_MAX_TICKS),
         ignore_sig(0),
@@ -143,15 +171,19 @@ struct RecordFlags {
         use_syscall_buffer(RecordSession::ENABLE_SYSCALL_BUF),
         syscall_buffer_size(0),
         print_trace_dir(-1),
+        output_trace_dir(""),
         use_file_cloning(true),
         use_read_cloning(true),
         bind_cpu(BIND_CPU),
         always_switch(false),
         chaos(false),
+        num_cores(0),
         wait_for_all(false),
         ignore_nested(false),
         scarce_fds(false),
-        setuid_sudo(false) {}
+        setuid_sudo(false),
+        copy_preload_src(false),
+        syscallbuf_desched_sig(SYSCALLBUF_DEFAULT_DESCHED_SIGNAL) {}
 };
 
 static void parse_signal_name(ParsedOption& opt) {
@@ -204,12 +236,17 @@ static bool parse_record_arg(vector<string>& args, RecordFlags& flags) {
     { 7, "disable-cpuid-features", HAS_PARAMETER },
     { 8, "disable-cpuid-features-ext", HAS_PARAMETER },
     { 9, "disable-cpuid-features-xsave", HAS_PARAMETER },
+    { 10, "num-cores", HAS_PARAMETER },
+    { 11, "trace-id", HAS_PARAMETER },
+    { 12, "copy-preload-src", NO_PARAMETER },
+    { 13, "syscall-buffer-sig", HAS_PARAMETER },
     { 'b', "force-syscall-buffer", NO_PARAMETER },
     { 'c', "num-cpu-ticks", HAS_PARAMETER },
     { 'h', "chaos", NO_PARAMETER },
     { 'i', "ignore-signal", HAS_PARAMETER },
     { 'n', "no-syscall-buffer", NO_PARAMETER },
     { 'p', "print-trace-dir", HAS_PARAMETER },
+    { 'o', "output-trace-dir", HAS_PARAMETER },
     { 's', "always-switch", NO_PARAMETER },
     { 't', "continue-through-signal", HAS_PARAMETER },
     { 'u', "cpu-unbound", NO_PARAMETER },
@@ -252,6 +289,9 @@ static bool parse_record_arg(vector<string>& args, RecordFlags& flags) {
       }
       flags.print_trace_dir = opt.int_value;
       break;
+    case 'o':
+      flags.output_trace_dir = opt.value;
+      break;
     case 0:
       flags.use_read_cloning = false;
       break;
@@ -313,6 +353,84 @@ static bool parse_record_arg(vector<string>& args, RecordFlags& flags) {
       flags.disable_cpuid_features.xsave_features_eax = bits[0];
       break;
     }
+    case 10: {
+      if (!opt.verify_valid_int(1, 128)) {
+        return false;
+      }
+      flags.num_cores = opt.int_value;
+      break;
+    }
+    case 11: {
+      const uint8_t SUM_GROUP_LENS[5] = { 8, 12, 16, 20, 32 };
+      /* Parse UUIDs from string form optionally with hypens */
+      uint8_t digit = 0; // This counts only hex digits (i.e. not hypens)
+      uint8_t group = 0;
+      uint8_t acc = 0;
+      unique_ptr<TraceUuid> buf(new TraceUuid);
+      auto it = opt.value.begin();
+      while (it < opt.value.end()) {
+        auto c = *it;
+
+        if (digit > SUM_GROUP_LENS[4]) {
+          return false;
+        }
+
+        if (digit % 2 == 0) {
+          // First digit of the byte.
+          if ('0' <= c && c <= '9') {
+            acc = c - '0';
+          } else if ('a' <= c && c <= 'f') {
+            acc = c - 'a' + 10;
+          } else if ('A' <= c && c <= 'F') {
+            acc = c - 'A' + 10;
+          } else if (c == '-') {
+            // Group delimiter.
+            if (SUM_GROUP_LENS[group] != digit) {
+              return false;
+            }
+            ++group;
+            ++it;
+            continue;
+          } else {
+            return false;
+          }
+        } else {
+          // Second digit of the byte.
+          acc <<= 4;
+          if ('0' <= c && c <= '9') {
+            acc += c - '0';
+          } else if ('a' <= c && c <= 'f') {
+            acc += c - 'a' + 10;
+          } else if ('A' <= c && c <= 'F') {
+            acc += c - 'A' + 10;
+          } else {
+            return false;
+          }
+
+          buf->bytes[digit / 2] = acc;
+        }
+
+        ++digit;
+        ++it;
+      }
+
+      if (SUM_GROUP_LENS[4] != digit) {
+        return false;
+      }
+
+      flags.trace_id.swap(buf);
+      break;
+    }
+    case 12:
+      flags.copy_preload_src = true;
+      break;
+    case 13:
+      parse_signal_name(opt);
+      if (!opt.verify_valid_int(1, _NSIG - 1)) {
+        return false;
+      }
+      flags.syscallbuf_desched_sig = opt.int_value;
+      break;
     case 's':
       flags.always_switch = true;
       break;
@@ -372,7 +490,10 @@ static void install_signal_handlers(void) {
   sigaction(SIGTERM, &sa, nullptr);
 
   sa.sa_handler = SIG_IGN;
+  sigaction(SIGHUP, &sa, nullptr);
   sigaction(SIGINT, &sa, nullptr);
+  sigaction(SIGABRT, &sa, nullptr);
+  sigaction(SIGQUIT, &sa, nullptr);
 }
 
 static void setup_session_from_flags(RecordSession& session,
@@ -380,6 +501,11 @@ static void setup_session_from_flags(RecordSession& session,
   session.scheduler().set_max_ticks(flags.max_ticks);
   session.scheduler().set_always_switch(flags.always_switch);
   session.set_enable_chaos(flags.chaos);
+  if (flags.num_cores) {
+    // Set the number of cores reported, possibly overriding the chaos mode
+    // setting.
+    session.set_num_cores(flags.num_cores);
+  }
   session.set_use_read_cloning(flags.use_read_cloning);
   session.set_use_file_cloning(flags.use_file_cloning);
   session.set_ignore_sig(flags.ignore_sig);
@@ -406,12 +532,54 @@ void force_close_record_session() {
   }
 }
 
+static void copy_preload_sources_to_trace(const string& trace_dir) {
+  string files_dir = trace_dir + "/files.rr";
+  mkdir(files_dir.c_str(), 0700);
+  pid_t pid;
+  string dest_path = files_dir + "/librrpreload.zip";
+  string src_path = resource_path() + "share/rr/src";
+  char zip[] = "zip";
+  char r[] = "-r";
+  char j[] = "-j";
+  char* argv[] = {
+    zip, r, j,
+    const_cast<char*>(dest_path.c_str()),
+    const_cast<char*>(src_path.c_str()),
+    NULL
+  };
+  posix_spawn_file_actions_t actions;
+  posix_spawn_file_actions_init(&actions);
+  posix_spawn_file_actions_addopen(&actions, STDOUT_FILENO, "/dev/null", O_RDONLY, 0);
+  posix_spawn_file_actions_addopen(&actions, STDERR_FILENO, "/dev/null", O_RDONLY, 0);
+  int ret = posix_spawnp(&pid, argv[0], &actions, NULL, argv, environ);
+  if (ret) {
+    FATAL() << "Can't spawn 'zip'";
+  }
+  posix_spawn_file_actions_destroy(&actions);
+  int status;
+  waitpid(pid, &status, 0);
+  LOG(info) << "Got zip status " << WaitStatus(status);
+}
+
+static void save_rr_git_revision(const string& trace_dir) {
+  string files_dir = trace_dir + "/files.rr";
+  mkdir(files_dir.c_str(), 0700);
+  string dest_path = files_dir + "/rr_git_revision";
+  ScopedFd fd(dest_path.c_str(), O_CREAT | O_WRONLY, 0600);
+  ssize_t written = write(fd, GIT_REVISION, sizeof(GIT_REVISION) - 1);
+  if (written != sizeof(GIT_REVISION) - 1) {
+    FATAL() << "Can't write GIT_REVISION";
+  }
+}
+
 static WaitStatus record(const vector<string>& args, const RecordFlags& flags) {
   LOG(info) << "Start recording...";
 
   auto session = RecordSession::create(
       args, flags.extra_env, flags.disable_cpuid_features,
-      flags.use_syscall_buffer, flags.bind_cpu);
+      flags.use_syscall_buffer, flags.syscallbuf_desched_sig,
+      flags.bind_cpu, flags.output_trace_dir,
+      flags.trace_id.get());
   setup_session_from_flags(*session, flags);
 
   static_session = session.get();
@@ -422,6 +590,12 @@ static WaitStatus record(const vector<string>& args, const RecordFlags& flags) {
     write_all(flags.print_trace_dir, "\n", 1);
   }
 
+  if (flags.copy_preload_src) {
+    const string& dir = session->trace_writer().dir();
+    copy_preload_sources_to_trace(dir);
+    save_rr_git_revision(dir);
+  }
+
   // Install signal handlers after creating the session, to ensure they're not
   // inherited by the tracee.
   install_signal_handlers();
@@ -440,8 +614,8 @@ static WaitStatus record(const vector<string>& args, const RecordFlags& flags) {
 
   switch (step_result.status) {
     case RecordSession::STEP_CONTINUE:
-      // SIGINT or something like that interrupted us.
-      return WaitStatus::for_fatal_sig(SIGINT);
+      // SIGTERM interrupted us.
+      return WaitStatus::for_fatal_sig(SIGTERM);
 
     case RecordSession::STEP_EXITED:
       return step_result.exit_status;
