@@ -128,7 +128,6 @@ static bool handle_ptrace_exit_event(RecordTask* t) {
     LOG(debug) << "stable exit";
   } else {
     if (!t->may_be_blocked()) {
-      // Record any progress this task made before it died. A running task
       // might have been hit by a SIGKILL or a SECCOMP_RET_KILL, in which case
       // there might be some execution since its last recorded event that we
       // need to replay.
@@ -162,7 +161,12 @@ static bool handle_ptrace_exit_event(RecordTask* t) {
           t->record_event(event);
         }
       } else {
-        t->record_event(Event::sched());
+        // Don't try to reset the syscallbuf here. The task may be exiting
+        // while in arbitrary syscallbuf code. And of course, because it's
+        // exiting, it doesn't matter if we don't reset the syscallbuf.
+        // XXX flushing the syscallbuf may be risky too...
+        t->record_event(Event::sched(), RecordTask::FLUSH_SYSCALLBUF,
+                        RecordTask::DONT_RESET_SYSCALLBUF);
       }
     }
     LOG(warn)
@@ -426,6 +430,9 @@ static void handle_seccomp_errno(RecordTask* t,
 
   if (!t->is_in_untraced_syscall()) {
     t->push_syscall_event(syscallno);
+    // Note that the syscall failed. prepare_clone() needs to know
+    // this during replay of the syscall entry.
+    t->ev().Syscall().failed_during_preparation = true;
     note_entering_syscall(t);
   }
 
@@ -444,7 +451,8 @@ bool RecordSession::handle_ptrace_event(RecordTask** t_ptr,
   *did_enter_syscall = false;
 
   RecordTask* t = *t_ptr;
-  if (t->status().group_stop()) {
+  if (t->status().group_stop() || t->has_stashed_group_stop()) {
+    t->clear_stashed_group_stop();
     last_task_switchable = ALLOW_SWITCH;
     step_state->continue_type = DONT_CONTINUE;
     return true;
@@ -700,7 +708,7 @@ static void advance_to_disarm_desched_syscall(RecordTask* t) {
     // we arm the desched signal before we restart a blocking syscall, which
     // completes successfully, then we disarm, then we see a desched signal
     // here.
-    if (SYSCALLBUF_DESCHED_SIGNAL == sig) {
+    if (t->session().syscallbuf_desched_sig() == sig) {
       continue;
     }
     if (sig && sig == old_sig) {
@@ -888,6 +896,7 @@ void RecordSession::syscall_state_changed(RecordTask* t,
       last_task_switchable = t->ev().Syscall().switchable =
           rec_prepare_syscall(t);
       t->record_event(t->ev(), RecordTask::FLUSH_SYSCALLBUF,
+                      RecordTask::ALLOW_RESET_SYSCALLBUF,
                       &t->ev().Syscall().regs);
 
       debug_exec_state("after cont", t);
@@ -1175,7 +1184,7 @@ static bool preinject_signal(RecordTask* t) {
     // Always send SYSCALLBUF_DESCHED_SIGNAL because other signals (except
     // TIME_SLICE_SIGNAL) will be blocked by
     // RecordTask::will_resume_execution().
-    t->tgkill(SYSCALLBUF_DESCHED_SIGNAL);
+    t->tgkill(t->session().syscallbuf_desched_sig());
 
     /* Now singlestep the task until we're in a signal-stop for the signal
      * we've just sent. We must absorb and forget that signal here since we
@@ -1197,7 +1206,7 @@ static bool preinject_signal(RecordTask* t) {
       return false;
     }
 
-    ASSERT(t, t->stop_sig() == SYSCALLBUF_DESCHED_SIGNAL)
+    ASSERT(t, t->stop_sig() == t->session().syscallbuf_desched_sig())
         << "Expected SYSCALLBUF_DESCHED_SIGNAL, got " << t->status();
     /* We're now in a signal-stop */
   }
@@ -1238,14 +1247,14 @@ static bool inject_handled_signal(RecordTask* t) {
     t->invalidate_sigmask();
     // Repeat injection if we got a desched signal. We observe in Linux 4.14.12
     // that we get SYSCALLBUF_DESCHED_SIGNAL here once in a while.
-  } while (t->stop_sig() == SYSCALLBUF_DESCHED_SIGNAL);
+  } while (t->stop_sig() == t->session().syscallbuf_desched_sig());
 
   if (t->stop_sig() == SIGSEGV) {
     // Constructing the signal handler frame must have failed. The kernel will
     // kill the process after this. Stash the signal and make sure
     // we know to treat it as fatal when we inject it. Also disable the
     // signal handler to match what the kernel does.
-    t->set_sig_handler_default(SIGSEGV);
+    t->did_set_sig_handler_default(SIGSEGV);
     t->stash_sig();
     t->thread_group()->received_sigframe_SIGSEGV = true;
     return false;
@@ -1349,9 +1358,9 @@ void RecordSession::signal_state_changed(RecordTask* t, StepState* step_state) {
       }
 
       // We record this data even if sigframe_size is zero to simplify replay.
-      // Stop recording data if we run off the end of a writeable mapping.
+      // Stop recording data if we run off the end of a writable mapping.
       // Our sigframe size is conservative so we need to do this.
-      t->record_remote_writeable(t->regs().sp(), sigframe_size);
+      t->record_remote_writable(t->regs().sp(), sigframe_size);
 
       // This event is used by the replayer to set up the signal handler frame.
       // But if we don't have a handler, we don't want to record the event
@@ -1402,7 +1411,8 @@ void RecordSession::signal_state_changed(RecordTask* t, StepState* step_state) {
         can_switch = ALLOW_SWITCH;
       }
 
-      t->record_event(t->ev(), RecordTask::FLUSH_SYSCALLBUF, &r);
+      t->record_event(t->ev(), RecordTask::FLUSH_SYSCALLBUF,
+                      RecordTask::ALLOW_RESET_SYSCALLBUF, &r);
       // Don't actually set_regs(r), the kernel does these modifications.
 
       // Only inject fatal signals. Non-fatal signals with signal handlers
@@ -1464,7 +1474,7 @@ bool RecordSession::handle_signal_event(RecordTask* t, StepState* step_state) {
   // was blocked now, before we update our cached sigmask.
   SignalBlocked signal_was_blocked =
       t->is_sig_blocked(sig) ? SIG_BLOCKED : SIG_UNBLOCKED;
-  if (deterministic || sig == SYSCALLBUF_DESCHED_SIGNAL) {
+  if (deterministic || sig == t->session().syscallbuf_desched_sig()) {
     // Don't stash these signals; deliver them immediately.
     // We don't want them to be reordered around other signals.
     // invalidate_sigmask() must not be called before we reach handle_signal!
@@ -1539,7 +1549,7 @@ bool RecordSession::handle_signal_event(RecordTask* t, StepState* step_state) {
 bool RecordSession::process_syscall_entry(RecordTask* t, StepState* step_state,
                                           RecordResult* step_result,
                                           SupportedArch syscall_arch) {
-  if (t->has_stashed_sig_not_synthetic_SIGCHLD()) {
+  if (const siginfo_t* si = t->stashed_sig_not_synthetic_SIGCHLD()) {
     // The only four cases where we allow a stashed signal to be pending on
     // syscall entry are:
     // -- the signal is a ptrace-related signal, in which case if it's generated
@@ -1560,7 +1570,9 @@ bool RecordSession::process_syscall_entry(RecordTask* t, StepState* step_state,
                t->ip() ==
                    t->vm()
                        ->privileged_traced_syscall_ip()
-                       .increment_by_syscall_insn_length(t->arch()));
+                       .increment_by_syscall_insn_length(t->arch()))
+      << "Stashed signal pending on syscall entry when it shouldn't be: "
+      << *si << "; IP=" << t->ip();
   }
 
   // We just entered a syscall.
@@ -1672,6 +1684,15 @@ bool RecordSession::prepare_to_inject_signal(RecordTask* t,
     }
   }
 
+  if (sig->deterministic == DETERMINISTIC_SIG &&
+      sig->siginfo.si_signo == SIGSYS &&
+      t->is_sig_blocked(sig->siginfo.si_signo) == SIG_BLOCKED) {
+    // Our synthesized deterministic SIGSYS (seccomp trap) needs to match the
+    // kernel behavior of unblocking the signal and resetting disposition to
+    // default.
+    t->unblock_signal(SIGSYS);
+    t->set_sig_handler_default(SIGSYS);
+  }
   switch (handle_signal(t, &si.linux_api, sig->deterministic, SIG_UNBLOCKED)) {
     case SIGNAL_PTRACE_STOP:
       // Emulated ptrace-stop. Don't run the task again yet.
@@ -1708,37 +1729,54 @@ bool RecordSession::prepare_to_inject_signal(RecordTask* t,
 }
 
 static string find_syscall_buffer_library() {
-  string lib_path = exe_directory() + "../lib/rr/";
+  string lib_path = resource_path() + "lib64/rr/";
   string file_name = lib_path + SYSCALLBUF_LIB_FILENAME;
-  if (access(file_name.c_str(), F_OK) != 0) {
-    // File does not exist. Assume install put it in LD_LIBRARY_PATH.
-    lib_path = "";
+  if (access(file_name.c_str(), F_OK) == 0) {
+    return lib_path;
   }
+  lib_path = resource_path() + "lib/rr/";
+  file_name = lib_path + SYSCALLBUF_LIB_FILENAME;
+  if (access(file_name.c_str(), F_OK) == 0) {
+    return lib_path;
+  }
+  // File does not exist. Assume install put it in LD_LIBRARY_PATH.
+  lib_path = "";
   return lib_path;
 }
 
-/**
- * Returns the name of the first dynamic library that |exe_file| depends on
- * that starts with |prefix|, or an empty string if there isn't one or
- * anything fails.
- */
-static string find_needed_library_starting_with(const string& exe_file,
-                                                const string& prefix) {
+struct ExeInfo {
+  ExeInfo() : has_asan_symbols(false) {}
+  // Empty if anything fails
+  string libasan_path;
+  bool has_asan_symbols;
+};
+
+static ExeInfo read_exe_info(const string& exe_file) {
+  ExeInfo ret;
   ScopedFd fd(exe_file.c_str(), O_RDONLY);
   if (!fd.is_open()) {
-    return string();
+    return ret;
   }
   ElfFileReader reader(fd);
+
   DynamicSection dynamic = reader.read_dynamic();
   for (auto& entry : dynamic.entries) {
     if (entry.tag == DT_NEEDED && entry.val < dynamic.strtab.size()) {
       const char* name = &dynamic.strtab[entry.val];
-      if (!strncmp(name, prefix.c_str(), prefix.size())) {
-        return string(name);
+      if (!strncmp(name, "libasan", 7)) {
+        ret.libasan_path = string(name);
       }
     }
   }
-  return string();
+
+  auto syms = reader.read_symbols(".dynsym", ".dynstr");
+  for (size_t i = 0; i < syms.size(); ++i) {
+    if (syms.is_name(i, "__asan_init")) {
+      ret.has_asan_symbols = true;
+    }
+  }
+
+  return ret;
 }
 
 static string lookup_by_path(const string& name) {
@@ -1775,7 +1813,11 @@ static string lookup_by_path(const string& name) {
 /*static*/ RecordSession::shr_ptr RecordSession::create(
     const vector<string>& argv, const vector<string>& extra_env,
     const DisableCPUIDFeatures& disable_cpuid_features,
-    SyscallBuffering syscallbuf, BindCPU bind_cpu) {
+    SyscallBuffering syscallbuf,
+    unsigned char syscallbuf_desched_sig,
+    BindCPU bind_cpu,
+    const string& output_trace_dir,
+    const TraceUuid* trace_id) {
   // The syscallbuf library interposes some critical
   // external symbols like XShmQueryExtension(), so we
   // preload it whether or not syscallbuf is enabled. Indicate here whether
@@ -1793,11 +1835,12 @@ static string lookup_by_path(const string& name) {
         buf[size] = 0;
         int val = atoi(buf);
         if (val > 1) {
-          FATAL() << "rr needs /proc/sys/kernel/perf_event_paranoid <= 1, but "
-                     "it is "
-                  << val << ".\nChange it to 1, or use 'rr record -n' (slow).\n"
-                  << "Consider putting 'kernel.perf_event_paranoid = 1' in "
-                     "/etc/sysctl.conf";
+          fprintf(stderr,
+                  "rr needs /proc/sys/kernel/perf_event_paranoid <= 1, but it is %d.\n"
+                  "Change it to 1, or use 'rr record -n' (slow).\n"
+                  "Consider putting 'kernel.perf_event_paranoid = 1' in /etc/sysctl.conf\n",
+                  val);
+          exit(1);
         }
       }
     }
@@ -1807,17 +1850,17 @@ static string lookup_by_path(const string& name) {
   env.insert(env.end(), extra_env.begin(), extra_env.end());
 
   string full_path = lookup_by_path(argv[0]);
+  ExeInfo exe_info = read_exe_info(full_path);
 
   // LD_PRELOAD the syscall interception lib
   string syscall_buffer_lib_path = find_syscall_buffer_library();
   if (!syscall_buffer_lib_path.empty()) {
     string ld_preload = "LD_PRELOAD=";
-    string libasan = find_needed_library_starting_with(full_path, "libasan");
-    if (!libasan.empty()) {
-      LOG(debug) << "Prepending " << libasan << " to LD_PRELOAD";
+    if (!exe_info.libasan_path.empty()) {
+      LOG(debug) << "Prepending " << exe_info.libasan_path << " to LD_PRELOAD";
       // Put an LD_PRELOAD entry for it before our preload library, because
       // it checks that it's loaded first
-      ld_preload += libasan + ":";
+      ld_preload += exe_info.libasan_path + ":";
     }
     // Our preload lib should come first if possible, because that will
     // speed up the loading of the other libraries. We supply a placeholder
@@ -1853,14 +1896,23 @@ static string lookup_by_path(const string& name) {
   // it is useless when running under rr.
   env.push_back("MOZ_GDB_SLEEP=0");
 
-  // OpenSSL uses RDRAND, but we can disable it. These bitmasks are inverted
-  // and ANDed with the results of CPUID. The number below is 2^62, which is the
-  // bit for RDRAND support.
-  env.push_back("OPENSSL_ia32cap=~4611686018427387904:~0");
+  // If we have CPUID faulting, don't use these environment hacks. We don't
+  // need them and the user might want to use them themselves for other reasons.
+  if (!Session::has_cpuid_faulting()) {
+    // OpenSSL uses RDRAND, but we can disable it. These bitmasks are inverted
+    // and ANDed with the results of CPUID. The number below is 2^62, which is the
+    // bit for RDRAND support.
+    env.push_back("OPENSSL_ia32cap=~4611686018427387904:~0");
+    // Disable Qt's use of RDRAND/RDSEED/RTM
+    env.push_back("QT_NO_CPU_FEATURE=rdrand rdseed rtm");
+  }
 
   shr_ptr session(
       new RecordSession(full_path, argv, env, disable_cpuid_features,
-                        syscallbuf, bind_cpu));
+                        syscallbuf, syscallbuf_desched_sig, bind_cpu,
+                        output_trace_dir, trace_id));
+  session->set_asan_active(!exe_info.libasan_path.empty() ||
+                           exe_info.has_asan_symbols);
   return session;
 }
 
@@ -1868,19 +1920,25 @@ RecordSession::RecordSession(const std::string& exe_path,
                              const std::vector<std::string>& argv,
                              const std::vector<std::string>& envp,
                              const DisableCPUIDFeatures& disable_cpuid_features,
-                             SyscallBuffering syscallbuf, BindCPU bind_cpu)
-    : trace_out(argv[0], choose_cpu(bind_cpu), has_cpuid_faulting_,
-                disable_cpuid_features),
+                             SyscallBuffering syscallbuf,
+                             int syscallbuf_desched_sig,
+                             BindCPU bind_cpu,
+                             const string& output_trace_dir,
+                             const TraceUuid* trace_id)
+    : trace_out(argv[0], choose_cpu(bind_cpu), output_trace_dir, ticks_semantics_),
       scheduler_(*this),
+      trace_id(trace_id),
       disable_cpuid_features_(disable_cpuid_features),
       ignore_sig(0),
       continue_through_sig(0),
       last_task_switchable(PREVENT_SWITCH),
       syscall_buffer_size_(1024 * 1024),
+      syscallbuf_desched_sig_(syscallbuf_desched_sig),
       use_syscall_buffer_(syscallbuf == ENABLE_SYSCALL_BUF),
       use_file_cloning_(true),
       use_read_cloning_(true),
       enable_chaos_(false),
+      asan_active_(false),
       wait_for_all_(false) {
   if (!has_cpuid_faulting() &&
       disable_cpuid_features.any_features_disabled()) {
@@ -1892,6 +1950,8 @@ RecordSession::RecordSession(const std::string& exe_path,
       Task::spawn(*this, error_fd, &tracee_socket_fd(), 
                   &tracee_socket_fd_number, trace_out,
                   exe_path, argv, envp));
+  // CPU affinity has been set.
+  trace_out.setup_cpuid_records(has_cpuid_faulting(), disable_cpuid_features_);
 
   initial_thread_group = t->thread_group();
   on_create(t);
@@ -2033,8 +2093,11 @@ void RecordSession::terminate_recording() {
   // This will write unstable exit events for all tasks.
   kill_all_tasks();
   t = nullptr; // t is now deallocated
+  close_trace_writer(TraceWriter::CLOSE_OK);
+}
 
-  trace_out.close();
+void RecordSession::close_trace_writer(TraceWriter::CloseStatus status) {
+  trace_out.close(status, trace_id.get());
 }
 
 Task* RecordSession::new_task(pid_t tid, pid_t, uint32_t serial,
@@ -2060,6 +2123,11 @@ RecordTask* RecordSession::find_task(const TaskUid& tuid) const {
   return static_cast<RecordTask*>(Session::find_task(tuid));
 }
 
+uint64_t RecordSession::rr_signal_mask() const {
+  return signal_bit(PerfCounters::TIME_SLICE_SIGNAL) |
+         signal_bit(syscallbuf_desched_sig_);
+}
+
 static const uint32_t CPUID_RDRAND_FLAG = 1 << 30;
 static const uint32_t CPUID_RTM_FLAG = 1 << 11;
 static const uint32_t CPUID_RDSEED_FLAG = 1 << 18;
