@@ -46,6 +46,10 @@ struct DisableCPUIDFeatures {
   uint32_t xsave_features_eax;
 };
 
+struct TraceUuid {
+  uint8_t bytes[16];
+};
+
 /** Encapsulates additional session state related to recording. */
 class RecordSession : public Session {
 public:
@@ -60,19 +64,26 @@ public:
       const std::vector<std::string>& extra_env,
       const DisableCPUIDFeatures& features,
       SyscallBuffering syscallbuf = ENABLE_SYSCALL_BUF,
-      BindCPU bind_cpu = BIND_CPU);
+      unsigned char syscallbuf_desched_sig = SIGPWR,
+      BindCPU bind_cpu = BIND_CPU,
+      const std::string& output_trace_dir = "",
+      const TraceUuid* trace_id = nullptr);
 
   const DisableCPUIDFeatures& disable_cpuid_features() const {
     return disable_cpuid_features_;
   }
   bool use_syscall_buffer() const { return use_syscall_buffer_; }
   size_t syscall_buffer_size() const { return syscall_buffer_size_; }
+  unsigned char syscallbuf_desched_sig() const { return syscallbuf_desched_sig_; }
   bool use_read_cloning() const { return use_read_cloning_; }
   bool use_file_cloning() const { return use_file_cloning_; }
   void set_ignore_sig(int sig) { ignore_sig = sig; }
   int get_ignore_sig() const { return ignore_sig; }
   void set_continue_through_sig(int sig) { continue_through_sig = sig; }
   int get_continue_through_sig() const { return continue_through_sig; }
+  void set_asan_active(bool active) { asan_active_ = active; }
+  bool asan_active() const { return asan_active_; }
+  uint64_t rr_signal_mask() const;
 
   enum RecordStatus {
     // Some execution was recorded. record_step() can be called again.
@@ -108,6 +119,12 @@ public:
    */
   void terminate_recording();
 
+  /**
+   * Close trace output without flushing syscall buffers or writing
+   * task exit/termination records to the trace.
+   */
+  void close_trace_writer(TraceWriter::CloseStatus status);
+
   virtual RecordSession* as_record() override { return this; }
 
   TraceWriter& trace_writer() { return trace_out; }
@@ -134,6 +151,9 @@ public:
   }
   bool enable_chaos() const { return enable_chaos_; }
 
+  void set_num_cores(int num_cores) {
+    scheduler().set_num_cores(num_cores);
+  }
   void set_use_read_cloning(bool enable) { use_read_cloning_ = enable; }
   void set_use_file_cloning(bool enable) { use_file_cloning_ = enable; }
   void set_syscall_buffer_size(size_t size) { syscall_buffer_size_ = size; }
@@ -159,12 +179,18 @@ public:
    */
   RecordTask* revive_task_for_exec(pid_t rec_tid);
 
+  virtual TraceStream* trace_stream() override { return &trace_out; }
+
 private:
   RecordSession(const std::string& exe_path,
                 const std::vector<std::string>& argv,
                 const std::vector<std::string>& envp,
                 const DisableCPUIDFeatures& features,
-                SyscallBuffering syscallbuf, BindCPU bind_cpu);
+                SyscallBuffering syscallbuf,
+                int syscallbuf_desched_sig,
+                BindCPU bind_cpu,
+                const std::string& output_trace_dir,
+                const TraceUuid* trace_id);
 
   virtual void on_create(Task* t) override;
 
@@ -194,12 +220,14 @@ private:
   Scheduler scheduler_;
   ThreadGroup::shr_ptr initial_thread_group;
   SeccompFilterRewriter seccomp_filter_rewriter_;
+  std::unique_ptr<const TraceUuid> trace_id;
 
   DisableCPUIDFeatures disable_cpuid_features_;
   int ignore_sig;
   int continue_through_sig;
   Switchable last_task_switchable;
   size_t syscall_buffer_size_;
+  unsigned char syscallbuf_desched_sig_;
   bool use_syscall_buffer_;
 
   bool use_file_cloning_;
@@ -208,10 +236,13 @@ private:
    * When true, try to increase the probability of finding bugs.
    */
   bool enable_chaos_;
+  bool asan_active_;
   /**
    * When true, wait for all tracees to exit before finishing recording.
    */
   bool wait_for_all_;
+
+  std::string output_trace_dir;
 };
 
 } // namespace rr
