@@ -187,6 +187,7 @@ RecordTask::RecordTask(RecordSession& session, pid_t _tid, uint32_t serial,
       tsc_mode(PR_TSC_ENABLE),
       cpuid_mode(1),
       stashed_signals_blocking_more_signals(false),
+      stashed_group_stop(false),
       break_at_syscallbuf_traced_syscalls(false),
       break_at_syscallbuf_untraced_syscalls(false),
       break_at_syscallbuf_final_instruction(false),
@@ -256,8 +257,8 @@ RecordTask::~RecordTask() {
   // is kill_all_tasks() in which case it's OK to just drop the last chunk of
   // execution. Trying to flush syscallbuf for an exiting task could be bad,
   // e.g. it could be in the middle of syscallbuf code that's supposed to be
-  // atomic.
-  record_event(Event::exit(), DONT_FLUSH_SYSCALLBUF);
+  // atomic. For the same reasons don't allow syscallbuf to be reset here.
+  record_event(Event::exit(), DONT_FLUSH_SYSCALLBUF, DONT_RESET_SYSCALLBUF);
 
   // We expect tasks to usually exit by a call to exit() or
   // exit_group(), so it's not helpful to warn about that.
@@ -406,6 +407,11 @@ template <typename Arch> static void do_preload_init_arch(RecordTask* t) {
   t->write_mem(cores_ptr, cores);
   t->record_local(cores_ptr, &cores);
 
+  auto desched_sig = t->session().syscallbuf_desched_sig();
+  auto desched_sig_ptr = REMOTE_PTR_FIELD(params.globals.rptr(), desched_sig);
+  t->write_mem(desched_sig_ptr, desched_sig);
+  t->record_local(desched_sig_ptr, &desched_sig);
+
   uint64_t random_seed;
   do {
     random_seed = rand() | (uint64_t(rand()) << 32);
@@ -461,11 +467,12 @@ template <typename Arch> void RecordTask::init_buffers_arch() {
     args.syscallbuf_ptr = syscallbuf_child;
     desched_fd_child = args.desched_counter_fd;
     // Prevent the child from closing this fd
-    fds->add_monitor(desched_fd_child, new PreserveFileMonitor());
+    fds->add_monitor(this, desched_fd_child, new PreserveFileMonitor());
     desched_fd = remote.retrieve_fd(desched_fd_child);
 
     auto record_in_trace = trace_writer().write_mapped_region(
         this, syscallbuf_km, syscallbuf_km.fake_stat(),
+        vector<TraceRemoteFd>(),
         TraceWriter::RR_BUFFER_MAPPING);
     ASSERT(this, record_in_trace == TraceWriter::DONT_RECORD_IN_TRACE);
 
@@ -489,7 +496,7 @@ template <typename Arch> void RecordTask::init_buffers_arch() {
           // Prevent the child from closing this fd. We're going to close it
           // ourselves and we don't want the child closing it and then reopening
           // its own file with this fd.
-          fds->add_monitor(cloned_file_data_fd_child,
+          fds->add_monitor(this, cloned_file_data_fd_child,
                            new PreserveFileMonitor());
           remote.infallible_syscall(syscall_number_for_close(arch()),
                                     cloned_file_data);
@@ -502,7 +509,7 @@ template <typename Arch> void RecordTask::init_buffers_arch() {
     args.syscallbuf_size = 0;
   }
   args.scratch_buf = scratch_ptr;
-  args.scratch_size = scratch_size;
+  args.usable_scratch_size = usable_scratch_size();
 
   // Return the mapped buffers to the child.
   write_mem(child_args, args);
@@ -598,8 +605,7 @@ void RecordTask::will_resume_execution(ResumeRequest, WaitRequest,
     // because blocking it seems to cause problems for some hardware/kernel
     // configurations (see https://github.com/mozilla/rr/issues/1979),
     // causing them to stop counting events.
-    sig_set_t sigset = ~(signal_bit(SYSCALLBUF_DESCHED_SIGNAL) |
-                         signal_bit(PerfCounters::TIME_SLICE_SIGNAL));
+    sig_set_t sigset = ~session().rr_signal_mask();
     if (sig) {
       // We're injecting a signal, so make sure that signal is unblocked.
       sigset &= ~signal_bit(sig);
@@ -679,8 +685,8 @@ void RecordTask::did_wait() {
 
   if (stashed_signals_blocking_more_signals) {
     // Saved 'blocked_sigs' must still be correct regardless of syscallbuf
-    // state, because we do not allow stashed_signals_blocking_more_signals to
-    // hold across syscalls (traced or untraced) that change the signal mask.
+    // state, because we do not allow stashed_signals_blocking_more_signals
+    // to hold across syscalls (traced or untraced) that change the signal mask.
     ASSERT(this, !blocked_sigs_dirty);
     xptrace(PTRACE_SETSIGMASK, remote_ptr<void>(8), &blocked_sigs);
   } else if (syscallbuf_child) {
@@ -1057,7 +1063,7 @@ void RecordTask::signal_delivered(int sig) {
         if (h.disposition() == SIGNAL_HANDLER) {
           break;
         }
-      // Fall through...
+        RR_FALLTHROUGH;
       case SIGSTOP:
         // All threads in the process are stopped.
         for (Task* t : thread_group()->task_set()) {
@@ -1159,11 +1165,17 @@ void RecordTask::update_sigaction(const Registers& regs) {
 }
 
 sig_set_t RecordTask::read_sigmask_from_process() {
-  sig_set_t mask;
-  long ret = fallible_ptrace(PTRACE_GETSIGMASK,
-                             remote_ptr<void>(sizeof(sig_set_t)), &mask);
-  if (ret >= 0) {
-    return mask;
+  // During syscall interruptions, PTRACE_GETSIGMASK may return the sigmask that is going
+  // to be restored, not the kernel's current (internal) sigmask, which is what
+  // /proc/.../status reports. Always go with what /proc/.../status reports. See
+  // https://github.com/torvalds/linux/commit/fcfc2aa0185f4a731d05a21e9f359968fdfd02e7
+  if (!at_may_restart_syscall()) {
+    sig_set_t mask;
+    long ret = fallible_ptrace(PTRACE_GETSIGMASK,
+                               remote_ptr<void>(sizeof(sig_set_t)), &mask);
+    if (ret >= 0) {
+      return mask;
+    }
   }
 
   auto results = read_proc_status_fields(tid, "SigBlk");
@@ -1180,7 +1192,38 @@ sig_set_t RecordTask::get_sigmask() {
   return blocked_sigs;
 }
 
+void RecordTask::unblock_signal(int sig) {
+  sig_set_t mask = get_sigmask();
+  mask &= ~signal_bit(sig);
+  int ret = fallible_ptrace(PTRACE_SETSIGMASK, remote_ptr<void>(8), &mask);
+  if (ret < 0) {
+    if (errno == EIO) {
+      FATAL() << "PTRACE_SETSIGMASK not supported; rr requires Linux kernel >= 3.11";
+    }
+    ASSERT(this, errno == EINVAL);
+  } else {
+    LOG(debug) << "Set signal mask to block all signals (bar "
+               << "SYSCALLBUF_DESCHED_SIGNAL/TIME_SLICE_SIGNAL) while we "
+               << " have a stashed signal";
+  }
+  invalidate_sigmask();
+}
+
 void RecordTask::set_sig_handler_default(int sig) {
+  did_set_sig_handler_default(sig);
+  // This could happen during a syscallbuf untraced syscall. In that case
+  // our remote syscall here could trigger a desched signal if that event
+  // is armed, making progress impossible. Disarm the event now.
+  disarm_desched_event(this);
+  AutoRemoteSyscalls remote(this);
+  Sighandler& h = sighandlers->get(sig);
+  AutoRestoreMem mem(remote, h.sa.data(), h.sa.size());
+  remote.infallible_syscall(syscall_number_for_rt_sigaction(arch()),
+      sig, mem.get().as_int(), nullptr,
+      sigaction_sigset_size(arch()));
+}
+
+void RecordTask::did_set_sig_handler_default(int sig) {
   Sighandler& h = sighandlers->get(sig);
   reset_handler(&h, arch());
 }
@@ -1228,7 +1271,7 @@ void RecordTask::stash_sig() {
   int sig = stop_sig();
   ASSERT(this, sig);
   // Callers should avoid passing SYSCALLBUF_DESCHED_SIGNAL in here.
-  ASSERT(this, sig != SYSCALLBUF_DESCHED_SIGNAL);
+  ASSERT(this, sig != session().syscallbuf_desched_sig());
   // multiple non-RT signals coalesce
   if (sig < SIGRTMIN) {
     for (auto it = stashed_signals.begin(); it != stashed_signals.end(); ++it) {
@@ -1255,14 +1298,20 @@ void RecordTask::stash_synthetic_sig(const siginfo_t& si,
   int sig = si.si_signo;
   DEBUG_ASSERT(sig);
   // Callers should avoid passing SYSCALLBUF_DESCHED_SIGNAL in here.
-  DEBUG_ASSERT(sig != SYSCALLBUF_DESCHED_SIGNAL);
+  DEBUG_ASSERT(sig != session().syscallbuf_desched_sig());
   // multiple non-RT signals coalesce
   if (sig < SIGRTMIN) {
     for (auto it = stashed_signals.begin(); it != stashed_signals.end(); ++it) {
       if (it->siginfo.si_signo == sig) {
-        LOG(debug) << "discarding stashed signal " << sig
-                   << " since we already have one pending";
-        return;
+        if (deterministic == DETERMINISTIC_SIG &&
+            it->deterministic == NONDETERMINISTIC_SIG) {
+          stashed_signals.erase(it);
+          break;
+        } else {
+          LOG(debug) << "discarding stashed signal " << sig
+                     << " since we already have one pending";
+          return;
+        }
       }
     }
   }
@@ -1284,13 +1333,13 @@ bool RecordTask::has_stashed_sig(int sig) const {
   return false;
 }
 
-bool RecordTask::has_stashed_sig_not_synthetic_SIGCHLD() const {
+const siginfo_t* RecordTask::stashed_sig_not_synthetic_SIGCHLD() const {
   for (auto it = stashed_signals.begin(); it != stashed_signals.end(); ++it) {
     if (!is_synthetic_SIGCHLD(it->siginfo)) {
-      return true;
+      return &it->siginfo;
     }
   }
-  return false;
+  return nullptr;
 }
 
 void RecordTask::pop_stash_sig(const StashedSignal* stashed) {
@@ -1506,8 +1555,8 @@ void RecordTask::record_remote(remote_ptr<void> addr, ssize_t num_bytes) {
   trace_writer().write_raw(rec_tid, buf.data(), num_bytes, addr);
 }
 
-void RecordTask::record_remote_writeable(remote_ptr<void> addr,
-                                         ssize_t num_bytes) {
+void RecordTask::record_remote_writable(remote_ptr<void> addr,
+                                        ssize_t num_bytes) {
   ASSERT(this, num_bytes >= 0);
 
   remote_ptr<void> p = addr;
@@ -1526,21 +1575,23 @@ void RecordTask::record_remote_writeable(remote_ptr<void> addr,
   record_remote(addr, num_bytes);
 }
 
-void RecordTask::record_remote_fallible(remote_ptr<void> addr,
-                                        ssize_t num_bytes) {
+ssize_t RecordTask::record_remote_fallible(remote_ptr<void> addr,
+                                          ssize_t num_bytes) {
   ASSERT(this, num_bytes >= 0);
 
-  if (record_remote_by_local_map(addr, num_bytes) != 0) {
-    return;
+  if (record_remote_by_local_map(addr, num_bytes)) {
+    return num_bytes;
   }
 
   vector<uint8_t> buf;
+  ssize_t nread = 0;
   if (!addr.is_null()) {
     buf.resize(num_bytes);
-    ssize_t nread = read_bytes_fallible(addr, num_bytes, buf.data());
+    nread = read_bytes_fallible(addr, num_bytes, buf.data());
     buf.resize(max<ssize_t>(0, nread));
   }
   trace_writer().write_raw(rec_tid, buf.data(), buf.size(), addr);
+  return nread;
 }
 
 void RecordTask::record_remote_even_if_null(remote_ptr<void> addr,
@@ -1658,6 +1709,7 @@ void RecordTask::maybe_reset_syscallbuf() {
 }
 
 void RecordTask::record_event(const Event& ev, FlushSyscallbuf flush,
+                              AllowSyscallbufReset reset,
                               const Registers* registers) {
   if (flush == FLUSH_SYSCALLBUF) {
     maybe_flush_syscallbuf();
@@ -1688,7 +1740,7 @@ void RecordTask::record_event(const Event& ev, FlushSyscallbuf flush,
   trace_writer().write_frame(this, ev, registers, extra_registers);
   LOG(debug) << "Wrote event " << ev << " for time " << current_time;
 
-  if (!ev.has_ticks_slop() && ev.type() != EV_EXIT) {
+  if (!ev.has_ticks_slop() && reset == ALLOW_RESET_SYSCALLBUF) {
     ASSERT(this, flush == FLUSH_SYSCALLBUF);
     // After we've output an event, it's safe to reset the syscallbuf (if not
     // explicitly delayed) since we will have exited the syscallbuf code that
@@ -1844,4 +1896,20 @@ void RecordTask::maybe_restore_original_syscall_registers() {
                    preload_thread_locals());
 }
 
+bool RecordTask::post_vm_clone(CloneReason reason, int flags, Task* origin) {
+  if (Task::post_vm_clone(reason, flags, origin)) {
+    KernelMapping preload_thread_locals_mapping =
+      vm()->mapping_of(AddressSpace::preload_thread_locals_start()).map;
+    auto mode = trace_writer().write_mapped_region(
+      this, preload_thread_locals_mapping,
+      preload_thread_locals_mapping.fake_stat(),
+      vector<TraceRemoteFd>(),
+      TraceWriter::RR_BUFFER_MAPPING);
+    ASSERT(this, mode == TraceWriter::DONT_RECORD_IN_TRACE);
+    return true;
+  }
+
+  return false;
+};
+
 } // namespace rr
