@@ -228,7 +228,7 @@ public:
   /**
    * Reset the signal handler for this signal to the default.
    */
-  void set_sig_handler_default(int sig);
+  void did_set_sig_handler_default(int sig);
 
   /**
    * Check that our status for |sig| matches what's in /proc/<pid>/status.
@@ -259,7 +259,7 @@ public:
   void stash_synthetic_sig(const siginfo_t& si,
                            SignalDeterministic deterministic);
   bool has_stashed_sig() const { return !stashed_signals.empty(); }
-  bool has_stashed_sig_not_synthetic_SIGCHLD() const;
+  const siginfo_t* stashed_sig_not_synthetic_SIGCHLD() const;
   bool has_stashed_sig(int sig) const;
   struct StashedSignal {
     StashedSignal(const siginfo_t& siginfo, SignalDeterministic deterministic)
@@ -271,6 +271,14 @@ public:
   void pop_stash_sig(const StashedSignal* stashed);
   void stashed_signal_processed();
 
+  /**
+   * If a group-stop occurs at an inconvenient time, stash it and
+   * process it later.
+   */
+  void stash_group_stop() { stashed_group_stop = true; }
+  void clear_stashed_group_stop() { stashed_group_stop = false; }
+  bool has_stashed_group_stop() const { return stashed_group_stop; }
+
   /**
    * Return true if the current state of this looks like the
    * interrupted syscall at the top of our event stack, if there
@@ -351,10 +359,10 @@ public:
   }
   // Record as much as we can of the bytes in this range. Will record only
   // contiguous mapped data starting at `addr`.
-  void record_remote_fallible(remote_ptr<void> addr, ssize_t num_bytes);
+  ssize_t record_remote_fallible(remote_ptr<void> addr, ssize_t num_bytes);
   // Record as much as we can of the bytes in this range. Will record only
   // contiguous mapped-writable data starting at `addr`.
-  void record_remote_writeable(remote_ptr<void> addr, ssize_t num_bytes);
+  void record_remote_writable(remote_ptr<void> addr, ssize_t num_bytes);
 
   // Simple helper that attempts to use the local mapping to record if one
   // exists
@@ -427,7 +435,15 @@ public:
      */
     DONT_FLUSH_SYSCALLBUF
   };
+  enum AllowSyscallbufReset {
+    ALLOW_RESET_SYSCALLBUF,
+    /* Pass this if it's safe to replay the event before we process the
+     * syscallbuf records.
+     */
+    DONT_RESET_SYSCALLBUF
+  };
   void record_event(const Event& ev, FlushSyscallbuf flush = FLUSH_SYSCALLBUF,
+                    AllowSyscallbufReset reset = ALLOW_RESET_SYSCALLBUF,
                     const Registers* registers = nullptr);
 
   bool is_fatal_signal(int sig, SignalDeterministic deterministic) const;
@@ -494,6 +510,14 @@ public:
    * Just get the signal mask of the process.
    */
   sig_set_t read_sigmask_from_process();
+  /**
+   * Unblock the signal for the process.
+   */
+  void unblock_signal(int sig);
+  /**
+   * Set the signal handler to default for the process.
+   */
+  void set_sig_handler_default(int sig);
 
   ~RecordTask();
 
@@ -544,6 +568,8 @@ private:
   /** Update the clear-tid futex to |tid_addr|. */
   void set_tid_addr(remote_ptr<int> tid_addr);
 
+  virtual bool post_vm_clone(CloneReason reason, int flags, Task* origin) override;
+
 public:
   Ticks ticks_at_last_recorded_syscall_exit;
 
@@ -667,6 +693,7 @@ public:
   // next opportunity.
   std::deque<StashedSignal> stashed_signals;
   bool stashed_signals_blocking_more_signals;
+  bool stashed_group_stop;
   bool break_at_syscallbuf_traced_syscalls;
   bool break_at_syscallbuf_untraced_syscalls;
   bool break_at_syscallbuf_final_instruction;
