@@ -123,16 +123,14 @@ static void check_xsave_compatibility(const TraceReader& trace_in) {
   }
 
   if (tracee_xcr0 != our_xcr0) {
-    if (tracee_xsavec) {
-      LOG(warn) << "Trace XCR0 value " << HEX(tracee_xcr0) << " != our XCR0 "
-          << "value " << HEX(our_xcr0) << "; Replay will fail if the tracee "
-          << "used plain XSAVE";
-    } else if (!Flags::get().suppress_environment_warnings) {
-      // Tracee may have used XSAVE instructions which write different components
-      // to XSAVE instructions executed on our CPU. This will cause divergence.
+    if (!Flags::get().suppress_environment_warnings) {
+      // If the tracee used XSAVE instructions which write different components
+      // to XSAVE instructions executed on our CPU, or examines XCR0 directly,
+      // This will cause divergence. The dynamic linker examines XCR0 so this
+      // is nearly guaranteed.
       cerr << "Trace XCR0 value " << HEX(tracee_xcr0) << " != our XCR0 "
           << "value " << HEX(our_xcr0) << "; Replay will probably fail "
-          << "because glibc dynamic loader uses XSAVE\n\n";
+          << "because glibc dynamic loader examines XCR0\n\n";
     }
   }
 
@@ -157,13 +155,16 @@ static void check_xsave_compatibility(const TraceReader& trace_in) {
   }
 }
 
-ReplaySession::ReplaySession(const std::string& dir)
+ReplaySession::ReplaySession(const std::string& dir, const Flags& flags)
     : emu_fs(EmuFs::create()),
       trace_in(dir),
       trace_frame(),
       current_step(),
       ticks_at_start_of_event(0),
+      flags_(flags),
       trace_start_time(0) {
+  ticks_semantics_ = trace_in.ticks_semantics();
+
   memset(&last_siginfo_, 0, sizeof(last_siginfo_));
   advance_to_next_trace_frame();
 
@@ -179,12 +180,10 @@ ReplaySession::ReplaySession(const std::string& dir)
         << "Trace was recorded on a machine with different CPUID values\n"
            "and CPUID faulting is not enabled; replay will not work.";
   }
-  if (has_cpuid_faulting() && trace_in.bound_to_cpu() >= 0) {
-    // The recorded trace was bound to a CPU, but we have CPUID faulting so
-    // we can bind to a different CPU and preserve CPUID values, so let's do
-    // that. This avoids problems if the tracees were bound to a CPU number
-    // that doesn't exist on this machine.
-    trace_in.set_bound_cpu(choose_cpu(BIND_CPU));
+  if (!PerfCounters::supports_ticks_semantics(ticks_semantics_)) {
+    CLEAN_FATAL()
+        << "Trace was recorded on a machine that defines ticks differently\n"
+           "to this machine; replay will not work.";
   }
 
   check_xsave_compatibility(trace_in);
@@ -199,7 +198,7 @@ ReplaySession::ReplaySession(const ReplaySession& other)
       ticks_at_start_of_event(other.ticks_at_start_of_event),
       cpuid_bug_detector(other.cpuid_bug_detector),
       last_siginfo_(other.last_siginfo_),
-      flags(other.flags),
+      flags_(other.flags_),
       trace_start_time(other.trace_start_time) {}
 
 ReplaySession::~ReplaySession() {
@@ -267,6 +266,7 @@ DiversionSession::shr_ptr ReplaySession::clone_diversion() {
              << " to DiversionSession...";
 
   DiversionSession::shr_ptr session(new DiversionSession());
+  session->ticks_semantics_ = ticks_semantics_;
   session->tracee_socket = tracee_socket;
   session->tracee_socket_fd_number = tracee_socket_fd_number;
   LOG(debug) << "  deepfork session is " << session.get();
@@ -282,8 +282,9 @@ Task* ReplaySession::new_task(pid_t tid, pid_t rec_tid, uint32_t serial,
   return new ReplayTask(*this, tid, rec_tid, serial, a);
 }
 
-/*static*/ ReplaySession::shr_ptr ReplaySession::create(const string& dir) {
-  shr_ptr session(new ReplaySession(dir));
+/*static*/ ReplaySession::shr_ptr ReplaySession::create(const string& dir,
+                                                        const ReplaySession::Flags& flags) {
+  shr_ptr session(new ReplaySession(dir, flags));
 
   // It doesn't really matter what we use for argv/env here, since
   // replay_syscall's process_execve is going to follow the recording and
@@ -303,6 +304,13 @@ Task* ReplaySession::new_task(pid_t tid, pid_t rec_tid, uint32_t serial,
   return session;
 }
 
+int ReplaySession::cpu_binding(TraceStream& trace) const {
+  if (flags_.cpu_unbound) {
+    return -1;
+  }
+  return Session::cpu_binding(trace);
+}
+
 void ReplaySession::advance_to_next_trace_frame() {
   if (trace_in.at_end()) {
     trace_frame = TraceFrame(trace_frame.time(), 0, Event::trace_termination(),
@@ -616,7 +624,8 @@ void ReplaySession::check_pending_sig(ReplayTask* t) {
  * encountered.
  *
  * If we return INCOMPLETE, callers need to recalculate the constraints and
- * tick_request and try again.
+ * tick_request and try again. We may return INCOMPLETE because we successfully
+ * processed a CPUID trap.
  */
 Completion ReplaySession::continue_or_step(ReplayTask* t,
                                            const StepConstraints& constraints,
@@ -697,7 +706,7 @@ static void guard_unexpected_signal(ReplayTask* t) {
   if (t->stop_sig()) {
     ASSERT(t, false) << "Replay got unrecorded signal "
                      << signal_name(t->stop_sig()) << " while awaiting signal";
-  } else {
+  } else if (t->status().is_syscall()) {
     ASSERT(t, false) << "Replay got unrecorded syscall "
                      << syscall_name(t->regs().original_syscallno(), t->arch())
                      << " while awaiting signal";
@@ -1026,7 +1035,8 @@ void ReplaySession::check_ticks_consistency(ReplayTask* t, const Event& ev) {
 }
 
 static bool treat_signal_event_as_deterministic(const SignalEvent& ev) {
-  return ev.deterministic == DETERMINISTIC_SIG;
+  return ev.deterministic == DETERMINISTIC_SIG &&
+    ev.siginfo.si_signo != SIGBUS;
 }
 
 /**
