@@ -3,6 +3,7 @@
 #include "ReplayTimeline.h"
 
 #include <math.h>
+#include <sstream>
 
 #include "core.h"
 #include "fast_forward.h"
@@ -18,6 +19,21 @@ ReplayTimeline::InternalMark::~InternalMark() {
   }
 }
 
+void ReplayTimeline::InternalMark::full_print(FILE* out) const {
+  ostringstream buf;
+  buf << proto.key;
+  fprintf(out, "{%s,regs:", buf.str().c_str());
+  proto.regs.print_register_file(out);
+  fputs(",return_addresses=[", out);
+  for (int i = 0; i < ReturnAddressList::COUNT; ++i) {
+    fprintf(out, "%p", (void*)proto.return_addresses.addresses[i].as_int());
+    if (i + 1 < ReturnAddressList::COUNT) {
+      fputc(',', out);
+    }
+  }
+  fputs("]}", out);
+}
+
 ostream& operator<<(ostream& s, const ReplayTimeline::MarkKey& o) {
   return s << "time:" << o.trace_time << ",ticks:" << o.ticks
            << ",st:" << o.step_key.as_int();
@@ -61,14 +77,11 @@ bool ReplayTimeline::less_than(const Mark& m1, const Mark& m2) {
   return false;
 }
 
-ReplayTimeline::ReplayTimeline(std::shared_ptr<ReplaySession> session,
-                               const ReplaySession::Flags& session_flags)
-    : session_flags(session_flags),
-      current(std::move(session)),
+ReplayTimeline::ReplayTimeline(std::shared_ptr<ReplaySession> session)
+    : current(std::move(session)),
       breakpoints_applied(false),
       reverse_execution_barrier_event(0) {
   current->set_visible_execution(false);
-  current->set_flags(session_flags);
 }
 
 ReplayTimeline::~ReplayTimeline() {
@@ -215,11 +228,23 @@ void ReplayTimeline::mark_after_singlestep(const Mark& from,
     for (size_t i = 0; i < mark_vector.size(); ++i) {
       if (mark_vector[i] == from.ptr) {
         if (i + 1 >= mark_vector.size() || mark_vector[i + 1] != m.ptr) {
+          ssize_t m_prev = -1;
           for (size_t j = 0; j < mark_vector.size(); ++j) {
             LOG(debug) << "  mark_vector[" << j << "] " << *mark_vector[j];
+            if (mark_vector[j] == m.ptr && j > 0) {
+              m_prev = j - 1;
+            }
+          }
+          if (m_prev >= 0) {
+            fprintf(stderr, "Probable previous-to-duplicated-state at %d:", (int)m_prev);
+            mark_vector[m_prev]->full_print(stderr);
+            fprintf(stderr, "Probable previous-to-duplicated-state at %d:", (int)i);
+            from.ptr->full_print(stderr);
+            fprintf(stderr, "Probable duplicated state at %d:", (int)m_prev + 1);
+            m.ptr->full_print(stderr);
           }
           ASSERT(result.break_status.task, false)
-              << " expected to find " << m << " at index " << i + 1;
+              << " Probable duplicated states leading to " << m << " at index " << i + 1;
         }
         break;
       }
@@ -305,10 +330,9 @@ void ReplayTimeline::seek_to_before_key(const MarkKey& key) {
       // We can use the current session, so do nothing.
     } else {
       // nowhere earlier to go, so restart from beginning.
-      current = ReplaySession::create(current->trace_reader().dir());
+      current = ReplaySession::create(current->trace_reader().dir(), current->flags());
       breakpoints_applied = false;
       current_at_or_after_mark = nullptr;
-      current->set_flags(session_flags);
     }
   } else {
     --it;
@@ -429,18 +453,20 @@ ReplayResult ReplayTimeline::replay_step_to_mark(
     ReplaySession::StepConstraints constraints =
         strategy.setup_step_constraints();
     constraints.ticks_target = mark.ptr->proto.key.ticks - 1;
-    result = current->replay_step(constraints);
-    bool approaching_ticks_target =
+    if (constraints.ticks_target > 0) {
+      result = current->replay_step(constraints);
+      bool approaching_ticks_target =
         result.break_status.approaching_ticks_target;
-    result.break_status.approaching_ticks_target = false;
-    // We can't be at the mark yet.
-    ASSERT(t, t->tick_count() < mark.ptr->proto.key.ticks);
-    // If there's a break indicated, we should return that to the
-    // caller without doing any more work
-    if (!approaching_ticks_target || result.break_status.any_break()) {
-      update_strategy_and_fix_watchpoint_quirk(strategy, constraints, result,
-                                               before);
-      return result;
+      result.break_status.approaching_ticks_target = false;
+      // We can't be at the mark yet.
+      ASSERT(t, t->tick_count() < mark.ptr->proto.key.ticks);
+      // If there's a break indicated, we should return that to the
+      // caller without doing any more work
+      if (!approaching_ticks_target || result.break_status.any_break()) {
+        update_strategy_and_fix_watchpoint_quirk(strategy, constraints, result,
+                                                 before);
+        return result;
+      }
     }
     // We may not have made any progress so we'll need to try another strategy
   }
@@ -645,7 +671,7 @@ bool ReplayTimeline::add_watchpoint(ReplayTask* t, remote_ptr<void> addr,
   }
   watchpoints.insert(
       make_tuple(t->vm()->uid(), addr, num_bytes, type, move(condition)));
-  no_watchpoints_hit_interval_start = no_watchpoints_hit_interval_start =
+  no_watchpoints_hit_interval_start = no_watchpoints_hit_interval_end =
       Mark();
   return true;
 }
