@@ -30,8 +30,7 @@ private:
   struct InternalMark;
 
 public:
-  ReplayTimeline(std::shared_ptr<ReplaySession> session,
-                 const ReplaySession::Flags& session_flags);
+  ReplayTimeline(std::shared_ptr<ReplaySession> session);
   ReplayTimeline() : breakpoints_applied(false) {}
   ~ReplayTimeline();
 
@@ -340,6 +339,7 @@ private:
     bool operator<(const std::shared_ptr<InternalMark> other);
 
     bool equal_states(ReplaySession& session) const;
+    void full_print(FILE* out) const;
 
     ReplayTimeline* owner;
     // Reuse ProtoMark to contain the MarkKey + Registers + ReturnAddressList.
@@ -450,8 +450,6 @@ private:
    */
   void evaluate_conditions(ReplayResult& result);
 
-  ReplaySession::Flags session_flags;
-
   ReplaySession::shr_ptr current;
   // current is known to be at or after this mark
   std::shared_ptr<InternalMark> current_at_or_after_mark;
