@@ -161,6 +161,11 @@ void Scheduler::set_enable_chaos(bool enable_chaos) {
   regenerate_affinity_mask();
 }
 
+void Scheduler::set_num_cores(int cores) {
+  pretend_num_cores_ = cores;
+  regenerate_affinity_mask();
+}
+
 RecordTask* Scheduler::get_next_task_with_same_priority(RecordTask* t) {
   if (!t || t->in_round_robin_queue) {
     return nullptr;
