@@ -16,6 +16,7 @@
 #include "AutoRemoteSyscalls.h"
 #include "EmuFs.h"
 #include "Flags.h"
+#include "PerfCounters.h"
 #include "Task.h"
 #include "ThreadGroup.h"
 #include "core.h"
@@ -42,10 +43,9 @@ Session::Session()
       tracee_socket_fd_number(0),
       next_task_serial_(1),
       syscall_seccomp_ordering_(PTRACE_SYSCALL_BEFORE_SECCOMP_UNKNOWN),
+      ticks_semantics_(PerfCounters::default_ticks_semantics()),
       done_initial_exec_(false),
-      visible_execution_(true),
-      has_cpuid_faulting_(!Flags::get().disable_cpuid_faulting &&
-                          cpuid_faulting_works()) {
+      visible_execution_(true) {
   LOG(debug) << "Session " << this << " created";
 }
 
@@ -63,9 +63,9 @@ Session::Session(const Session& other) {
   next_task_serial_ = other.next_task_serial_;
   done_initial_exec_ = other.done_initial_exec_;
   visible_execution_ = other.visible_execution_;
-  has_cpuid_faulting_ = other.has_cpuid_faulting_;
   tracee_socket = other.tracee_socket;
   tracee_socket_fd_number = other.tracee_socket_fd_number;
+  ticks_semantics_ = other.ticks_semantics_;
 }
 
 void Session::on_create(ThreadGroup* tg) { thread_group_map[tg->tguid()] = tg; }
@@ -576,7 +576,8 @@ const AddressSpace::Mapping& Session::recreate_shared_mmap(
     AutoRemoteSyscalls& remote, const AddressSpace::Mapping& m,
     PreserveContents preserve, MonitoredSharedMemory::shr_ptr&& monitored) {
   char name[PATH_MAX];
-  strncpy(name, m.map.fsname().c_str(), sizeof(name));
+  strncpy(name, m.map.fsname().c_str(), sizeof(name) - 1);
+  name[sizeof(name) - 1] = 0;
   uint32_t flags = m.flags;
   size_t size = m.map.size();
   void* preserved_data = preserve == PRESERVE_CONTENTS ? m.local_addr : nullptr;
@@ -737,4 +738,12 @@ void Session::copy_state_to(Session& dest, EmuFs& emu_fs, EmuFs& dest_emu_fs) {
   DEBUG_ASSERT(dest.vms().size() > 0);
 }
 
+bool Session::has_cpuid_faulting() {
+  return !Flags::get().disable_cpuid_faulting && cpuid_faulting_works();
+}
+
+int Session::cpu_binding(TraceStream& trace) const {
+  return trace.bound_to_cpu();
+}
+
 } // namespace rr
