@@ -319,13 +319,18 @@ public:
   PtraceSyscallBeforeSeccomp syscall_seccomp_ordering() {
     return syscall_seccomp_ordering_;
   }
-  bool has_cpuid_faulting() const { return has_cpuid_faulting_; }
 
+  static bool has_cpuid_faulting();
   static const char* rr_mapping_prefix();
 
   ScopedFd& tracee_socket_fd() { return *tracee_socket; }
   int tracee_fd_number() const { return tracee_socket_fd_number; }
 
+  virtual TraceStream* trace_stream() { return nullptr; }
+  TicksSemantics ticks_semantics() const { return ticks_semantics_; }
+
+  virtual int cpu_binding(TraceStream& trace) const;
+
 protected:
   Session();
   virtual ~Session();
@@ -366,6 +371,8 @@ protected:
 
   PtraceSyscallBeforeSeccomp syscall_seccomp_ordering_;
 
+  TicksSemantics ticks_semantics_;
+
   /**
    * True if we've done an exec so tracees are now in a state that will be
    * consistent across record and replay.
@@ -376,11 +383,6 @@ protected:
    * True while the execution of this session is visible to users.
    */
   bool visible_execution_;
-
-  /**
-   * True if ARCH_GET/SET_CPUID are supported on this system.
-   */
-  bool has_cpuid_faulting_;
 };
 
 } // namespace rr
