@@ -36,6 +36,7 @@
 
 #include "AutoRemoteSyscalls.h"
 #include "CPUIDBugDetector.h"
+#include "Flags.h"
 #include "MagicSaveDataMonitor.h"
 #include "PreserveFileMonitor.h"
 #include "RecordSession.h"
@@ -72,7 +73,7 @@ Task::Task(Session& session, pid_t _tid, pid_t _rec_tid, uint32_t serial,
       desched_fd_child(-1),
       // This will be initialized when the syscall buffer is.
       cloned_file_data_fd_child(-1),
-      hpc(_tid),
+      hpc(_tid, session.ticks_semantics()),
       tid(_tid),
       rec_tid(_rec_tid > 0 ? _rec_tid : _tid),
       syscallbuf_size(0),
@@ -99,10 +100,6 @@ Task::Task(Session& session, pid_t _tid, pid_t _rec_tid, uint32_t serial,
 void Task::destroy() {
   LOG(debug) << "task " << tid << " (rec:" << rec_tid << ") is dying ...";
 
-  // child_mem_fd needs to be valid since we won't be able to open
-  // it for futex_wait after we've detached.
-  ASSERT(this, as->mem_fd().is_open());
-
   fallible_ptrace(PTRACE_DETACH, nullptr, nullptr);
 
   // Subclasses can do something in their destructors after we've detached
@@ -178,6 +175,15 @@ struct stat Task::stat_fd(int fd) {
   return result;
 }
 
+struct stat Task::lstat_fd(int fd) {
+  char path[PATH_MAX];
+  snprintf(path, sizeof(path) - 1, "/proc/%d/fd/%d", tid, fd);
+  struct stat result;
+  auto ret = ::lstat(path, &result);
+  ASSERT(this, ret == 0);
+  return result;
+}
+
 ScopedFd Task::open_fd(int fd, int flags) {
   char path[PATH_MAX];
   snprintf(path, sizeof(path) - 1, "/proc/%d/fd/%d", tid, fd);
@@ -221,13 +227,14 @@ void Task::destroy_buffers() {
 void Task::unmap_buffers_for(
     AutoRemoteSyscalls& remote, Task* other,
     remote_ptr<struct syscallbuf_hdr> saved_syscallbuf_child) {
+  auto arch = remote.task()->arch();
   if (other->scratch_ptr) {
-    remote.infallible_syscall(syscall_number_for_munmap(arch()),
+    remote.infallible_syscall(syscall_number_for_munmap(arch),
                               other->scratch_ptr, other->scratch_size);
     vm()->unmap(this, other->scratch_ptr, other->scratch_size);
   }
   if (!saved_syscallbuf_child.is_null()) {
-    remote.infallible_syscall(syscall_number_for_munmap(arch()),
+    remote.infallible_syscall(syscall_number_for_munmap(arch),
                               saved_syscallbuf_child, other->syscallbuf_size);
     vm()->unmap(this, saved_syscallbuf_child, other->syscallbuf_size);
   }
@@ -237,20 +244,28 @@ void Task::unmap_buffers_for(
  * Must be idempotent.
  */
 void Task::close_buffers_for(AutoRemoteSyscalls& remote, Task* other) {
+  auto arch = remote.task()->arch();
   if (other->desched_fd_child >= 0) {
     if (session().is_recording()) {
-      remote.infallible_syscall(syscall_number_for_close(arch()),
+      remote.infallible_syscall(syscall_number_for_close(arch),
                                 other->desched_fd_child);
     }
     fds->did_close(other->desched_fd_child);
   }
   if (other->cloned_file_data_fd_child >= 0) {
-    remote.infallible_syscall(syscall_number_for_close(arch()),
+    remote.infallible_syscall(syscall_number_for_close(arch),
                               other->cloned_file_data_fd_child);
     fds->did_close(other->cloned_file_data_fd_child);
   }
 }
 
+void Task::emulate_jump(remote_code_ptr ip) {
+  Registers r = regs();
+  r.set_ip(ip);
+  set_regs(r);
+  ticks += PerfCounters::ticks_for_unconditional_indirect_branch(this);
+}
+
 bool Task::is_desched_event_syscall() {
   return is_ioctl_syscall(regs().original_syscallno(), arch()) &&
          desched_fd_child != -1 &&
@@ -582,6 +597,10 @@ void Task::enter_syscall() {
       continue;
     }
     ASSERT(this, !ptrace_event());
+    if (session().is_recording() && wait_status.group_stop()) {
+      static_cast<RecordTask*>(this)->stash_group_stop();
+      continue;
+    }
     if (!stop_sig()) {
       ASSERT(this, need_ptrace_syscall_event);
       need_ptrace_syscall_event = false;
@@ -593,7 +612,7 @@ void Task::enter_syscall() {
     }
     ASSERT(this, session().is_recording())
         << " got unexpected signal " << signal_name(stop_sig());
-    if (stop_sig() == SYSCALLBUF_DESCHED_SIGNAL) {
+    if (stop_sig() == session().as_record()->syscallbuf_desched_sig()) {
       continue;
     }
     static_cast<RecordTask*>(this)->stash_sig();
@@ -754,9 +773,9 @@ string Task::read_c_str(remote_ptr<char> child_addr) {
     // end_of_page) is mapped.
     remote_ptr<void> end_of_page = ceil_page_size(p + 1);
     ssize_t nbytes = end_of_page - p;
-    char buf[nbytes];
+    std::unique_ptr<char[]> buf(new char[nbytes]);
 
-    read_bytes_helper(p, nbytes, buf);
+    read_bytes_helper(p, nbytes, buf.get());
     for (int i = 0; i < nbytes; ++i) {
       if ('\0' == buf[i]) {
         return str;
@@ -820,7 +839,7 @@ uintptr_t Task::debug_status() {
 }
 
 void Task::set_debug_status(uintptr_t status) {
-  fallible_ptrace(PTRACE_POKEUSER, dr_user_word_offset(6), (void*)status);
+  set_debug_reg(6, status);
 }
 
 static bool is_singlestep_resume(ResumeRequest request) {
@@ -831,27 +850,33 @@ TrapReasons Task::compute_trap_reasons() {
   ASSERT(this, stop_sig() == SIGTRAP);
   TrapReasons reasons;
   uintptr_t status = debug_status();
+  reasons.singlestep = (status & DS_SINGLESTEP) != 0;
 
-  if (is_singlestep_resume(how_last_execution_resumed) &&
-      is_at_syscall_instruction(this, address_of_last_execution_resume) &&
-      ip() ==
-          address_of_last_execution_resume +
-              syscall_instruction_length(arch())) {
-    // During replay we execute syscall instructions in certain cases, e.g.
-    // mprotect with syscallbuf. The kernel does not set DS_SINGLESTEP when we
-    // step over those instructions so we need to detect that here.
-    reasons.singlestep = true;
-  } else if (is_singlestep_resume(how_last_execution_resumed) &&
-             trapped_instruction_at(this, address_of_last_execution_resume) ==
-                 TrappedInstruction::CPUID &&
-             ip() ==
-                 address_of_last_execution_resume +
-                     trapped_instruction_len(TrappedInstruction::CPUID)) {
-    // Likewise we emulate CPUID instructions and must forcibly detect that
-    // here.
-    reasons.singlestep = true;
-  } else {
-    reasons.singlestep = (status & DS_SINGLESTEP) != 0;
+  if (is_singlestep_resume(how_last_execution_resumed)) {
+    if (is_at_syscall_instruction(this, address_of_last_execution_resume) &&
+        ip() ==
+            address_of_last_execution_resume +
+                syscall_instruction_length(arch())) {
+      // During replay we execute syscall instructions in certain cases, e.g.
+      // mprotect with syscallbuf. The kernel does not set DS_SINGLESTEP when we
+      // step over those instructions so we need to detect that here.
+      reasons.singlestep = true;
+    } else {
+      TrappedInstruction ti =
+        trapped_instruction_at(this, address_of_last_execution_resume);
+      if (ti == TrappedInstruction::CPUID &&
+          ip() == address_of_last_execution_resume +
+                       trapped_instruction_len(TrappedInstruction::CPUID)) {
+        // Likewise we emulate CPUID instructions and must forcibly detect that
+        // here.
+        reasons.singlestep = true;
+      } else if (ti == TrappedInstruction::INT3 &&
+          ip() == address_of_last_execution_resume +
+                       trapped_instruction_len(TrappedInstruction::INT3)) {
+        // INT3 instructions should also be turned into a singlestep here.
+        reasons.singlestep = true;
+      }
+    }
   }
 
   // In VMWare Player 6.0.4 build-2249910, 32-bit Ubuntu x86 guest,
@@ -1136,74 +1161,92 @@ static WatchBytesX86 num_bytes_to_dr_len(size_t num_bytes) {
   }
 }
 
+struct DebugControl {
+  uintptr_t dr0_local : 1;
+  uintptr_t dr0_global : 1;
+  uintptr_t dr1_local : 1;
+  uintptr_t dr1_global : 1;
+  uintptr_t dr2_local : 1;
+  uintptr_t dr2_global : 1;
+  uintptr_t dr3_local : 1;
+  uintptr_t dr3_global : 1;
+
+  uintptr_t ignored : 8;
+
+  WatchType dr0_type : 2;
+  WatchBytesX86 dr0_len : 2;
+  WatchType dr1_type : 2;
+  WatchBytesX86 dr1_len : 2;
+  WatchType dr2_type : 2;
+  WatchBytesX86 dr2_len : 2;
+  WatchType dr3_type : 2;
+  WatchBytesX86 dr3_len : 2;
+
+  void enable(size_t index, WatchBytesX86 size, WatchType type) {
+    switch (index) {
+#define CASE(_i)                                                  \
+      case _i:                                                    \
+        dr##_i##_local = 1;                                       \
+        dr##_i##_global = 0;                                      \
+        dr##_i##_type = type;                                     \
+        dr##_i##_len = size;                                      \
+        break
+      CASE(0);
+      CASE(1);
+      CASE(2);
+      CASE(3);
+#undef CASE
+      default:
+        FATAL() << "Invalid index";
+    }
+  }
+};
+
+static_assert(sizeof(DebugControl) == sizeof(uintptr_t),
+              "Can't pack DebugControl");
+
+union PackedDebugControl {
+  uintptr_t packed;
+  DebugControl ctl;
+};
+
 bool Task::set_debug_regs(const DebugRegs& regs) {
-  struct DebugControl {
-    uintptr_t packed() { return *(uintptr_t*)this; }
-
-    uintptr_t dr0_local : 1;
-    uintptr_t dr0_global : 1;
-    uintptr_t dr1_local : 1;
-    uintptr_t dr1_global : 1;
-    uintptr_t dr2_local : 1;
-    uintptr_t dr2_global : 1;
-    uintptr_t dr3_local : 1;
-    uintptr_t dr3_global : 1;
-
-    uintptr_t ignored : 8;
-
-    WatchType dr0_type : 2;
-    WatchBytesX86 dr0_len : 2;
-    WatchType dr1_type : 2;
-    WatchBytesX86 dr1_len : 2;
-    WatchType dr2_type : 2;
-    WatchBytesX86 dr2_len : 2;
-    WatchType dr3_type : 2;
-    WatchBytesX86 dr3_len : 2;
-  } dr7;
-  memset(&dr7, 0, sizeof(dr7));
-  static_assert(sizeof(DebugControl) == sizeof(uintptr_t),
-                "Can't pack DebugControl");
-
   // Reset the debug status since we're about to change the set
   // of programmed watchpoints.
-  ptrace_if_alive(PTRACE_POKEUSER, dr_user_word_offset(6), 0);
-  // Ensure that we clear the programmed watchpoints in case
-  // enabling one of them fails.  We guarantee atomicity to the
-  // caller.
-  ptrace_if_alive(PTRACE_POKEUSER, dr_user_word_offset(7), 0);
+  set_debug_reg(6, 0);
+
   if (regs.size() > NUM_X86_WATCHPOINTS) {
+    set_debug_reg(7, 0);
     return false;
   }
 
-  size_t dr = 0;
+  // Work around kernel bug https://bugzilla.kernel.org/show_bug.cgi?id=200965.
+  // For every watchpoint we're going to use, enable it with size 1.
+  // This will let us set the address freely without potentially triggering
+  // the kernel bug which will reject an unaligned address if the watchpoint
+  // is disabled but was non-size-1.
+  PackedDebugControl dr7;
+  dr7.packed = 0;
+  for (size_t i = 0; i < regs.size(); ++i) {
+    dr7.ctl.enable(i, BYTES_1, WATCH_EXEC);
+  }
+  set_debug_reg(7, dr7.packed);
+
+  size_t index = 0;
   for (auto reg : regs) {
-    if (fallible_ptrace(PTRACE_POKEUSER, dr_user_word_offset(dr),
-                        (void*)reg.addr.as_int())) {
+    if (!set_debug_reg(index, reg.addr.as_int())) {
+      set_debug_reg(7, 0);
       return false;
     }
-    switch (dr++) {
-#define CASE_ENABLE_DR(_dr7, _i, _reg)                                         \
-  case _i:                                                                     \
-    _dr7.dr##_i##_local = 1;                                                   \
-    _dr7.dr##_i##_type = _reg.type;                                            \
-    _dr7.dr##_i##_len = num_bytes_to_dr_len(_reg.num_bytes);                   \
-    break
-      CASE_ENABLE_DR(dr7, 0, reg);
-      CASE_ENABLE_DR(dr7, 1, reg);
-      CASE_ENABLE_DR(dr7, 2, reg);
-      CASE_ENABLE_DR(dr7, 3, reg);
-#undef CASE_ENABLE_DR
-      default:
-        FATAL() << "There's no debug register " << dr;
-    }
+    dr7.ctl.enable(index, num_bytes_to_dr_len(reg.num_bytes), reg.type);
+    ++index;
   }
-  return 0 == fallible_ptrace(PTRACE_POKEUSER, dr_user_word_offset(7),
-                              (void*)dr7.packed());
+  return set_debug_reg(7, dr7.packed);
 }
 
 uintptr_t Task::get_debug_reg(size_t regno) {
   errno = 0;
-  auto result =
+  long result =
       fallible_ptrace(PTRACE_PEEKUSER, dr_user_word_offset(regno), nullptr);
   if (errno == ESRCH) {
     return 0;
@@ -1211,8 +1254,10 @@ uintptr_t Task::get_debug_reg(size_t regno) {
   return result;
 }
 
-void Task::set_debug_reg(size_t regno, uintptr_t value) {
+bool Task::set_debug_reg(size_t regno, uintptr_t value) {
+  errno = 0;
   fallible_ptrace(PTRACE_POKEUSER, dr_user_word_offset(regno), (void*)value);
+  return errno == ESRCH || errno == 0;
 }
 
 static void set_thread_area(std::vector<struct user_desc>& thread_areas_,
@@ -1348,6 +1393,7 @@ void Task::wait(double interrupt_after_elapsed) {
       // some cases it doesn't return normally at all!
 
       // Fake a PTRACE_EVENT_EXIT for this task.
+      LOG(warn) << "Synthesizing PTRACE_EVENT_EXIT for zombie process " << tid;
       status = WaitStatus::for_ptrace_event(PTRACE_EVENT_EXIT);
       ret = tid;
       // XXX could this leave unreaped zombies lying around?
@@ -1375,6 +1421,8 @@ void Task::wait(double interrupt_after_elapsed) {
                                              "forgotten";
 
     // Turn this into a PTRACE_EXIT_EVENT.
+    LOG(warn) << "Synthesizing PTRACE_EVENT_EXIT for process " << tid
+        << " exited with " << status.exit_code();
     status = WaitStatus::for_ptrace_event(PTRACE_EVENT_EXIT);
   }
 
@@ -1622,7 +1670,7 @@ static void set_thread_area_from_clone(Task* t, remote_ptr<void> tls) {
 }
 
 template <typename Arch>
-static void setup_thread_locals_from_clone_arch(Task* t, Task* origin) {
+static void setup_preload_thread_locals_from_clone_arch(Task* t, Task* origin) {
   void* local_addr = preload_thread_locals_local_addr(*t->vm());
   if (local_addr) {
     t->activate_preload_thread_locals();
@@ -1635,8 +1683,8 @@ static void setup_thread_locals_from_clone_arch(Task* t, Task* origin) {
   }
 }
 
-static void setup_thread_locals_from_clone(Task* t, Task* origin) {
-  RR_ARCH_FUNCTION(setup_thread_locals_from_clone_arch, t->arch(), t, origin);
+void Task::setup_preload_thread_locals_from_clone(Task* origin) {
+  RR_ARCH_FUNCTION(setup_preload_thread_locals_from_clone_arch, this->arch(), this, origin);
 }
 
 Task* Task::clone(CloneReason reason, int flags, remote_ptr<void> stack,
@@ -1742,15 +1790,22 @@ Task* Task::clone(CloneReason reason, int flags, remote_ptr<void> stack,
     }
   }
 
+  t->post_vm_clone(reason, flags, this);
+
+  return t;
+}
+
+bool Task::post_vm_clone(CloneReason reason, int flags, Task* origin) {
+  bool created_preload_thread_locals_mapping = false;
   if (!(CLONE_SHARE_VM & flags)) {
-    t->as->post_vm_clone(t);
+    created_preload_thread_locals_mapping = this->as->post_vm_clone(this);
   }
 
   if (reason == TRACEE_CLONE) {
-    setup_thread_locals_from_clone(t, this);
+    setup_preload_thread_locals_from_clone(origin);
   }
 
-  return t;
+  return created_preload_thread_locals_mapping;
 }
 
 Task* Task::os_fork_into(Session* session) {
@@ -1931,7 +1986,7 @@ long Task::fallible_ptrace(int request, remote_ptr<void> addr, void* data) {
   return ptrace(__ptrace_request(request), tid, addr, data);
 }
 
-void Task::open_mem_fd() {
+bool Task::open_mem_fd() {
   // Use ptrace to read/write during open_mem_fd
   as->set_mem_fd(ScopedFd());
 
@@ -1942,28 +1997,39 @@ void Task::open_mem_fd() {
   static const char path[] = "/proc/self/mem";
 
   AutoRemoteSyscalls remote(this);
-  long remote_fd;
+  int remote_fd;
   {
     AutoRestoreMem remote_path(remote, (const uint8_t*)path, sizeof(path));
-    // skip leading '/' since we want the path to be relative to the root fd
-    remote_fd =
-        remote.syscall(syscall_number_for_openat(arch()),
-                       RR_RESERVED_ROOT_DIR_FD, remote_path.get() + 1, O_RDWR);
+    if (remote_path.get()) {
+      // skip leading '/' since we want the path to be relative to the root fd
+      remote_fd =
+          remote.syscall(syscall_number_for_openat(arch()),
+                        RR_RESERVED_ROOT_DIR_FD, remote_path.get() + 1, O_RDWR);
+    } else {
+      remote_fd = -ESRCH;
+    }
   }
-  if (remote_fd < 0) {
-    // This can happen when a process fork()s after setuid; it can no longer
-    // open its own /proc/self/mem. Hopefully we can read the child's
-    // mem file in this case (because rr is probably running as root).
-    char buf[PATH_MAX];
-    sprintf(buf, "/proc/%d/mem", tid);
-    ScopedFd fd(buf, O_RDWR);
-    ASSERT(this, fd.is_open());
-    as->set_mem_fd(move(fd));
-  } else {
-    as->set_mem_fd(remote.retrieve_fd(remote_fd));
-    ASSERT(this, as->mem_fd().is_open());
-    remote.infallible_syscall(syscall_number_for_close(arch()), remote_fd);
+  ScopedFd fd;
+  if (remote_fd != -ESRCH) {
+    if (remote_fd < 0) {
+      // This can happen when a process fork()s after setuid; it can no longer
+      // open its own /proc/self/mem. Hopefully we can read the child's
+      // mem file in this case (because rr is probably running as root).
+      char buf[PATH_MAX];
+      sprintf(buf, "/proc/%d/mem", tid);
+      fd = ScopedFd(buf, O_RDWR);
+    } else {
+      fd = remote.retrieve_fd(remote_fd);
+      // Leak fd if the syscall fails due to the task being SIGKILLed unexpectedly
+      remote.syscall(syscall_number_for_close(arch()), remote_fd);
+    }
   }
+  if (!fd.is_open()) {
+    LOG(info) << "Can't retrieve mem fd for " << tid << "; process no longer exists?";
+    return false;
+  }
+  as->set_mem_fd(move(fd));
+  return true;
 }
 
 void Task::open_mem_fd_if_needed() {
@@ -2120,7 +2186,9 @@ ssize_t Task::read_bytes_fallible(remote_ptr<void> addr, ssize_t buf_size,
     // that's different than the one we see after reopening the
     // fd, after exec.
     if (0 == nread && 0 == all_read && 0 == errno) {
-      open_mem_fd();
+      if (!open_mem_fd()) {
+        return 0;
+      }
       continue;
     }
     if (nread <= 0) {
@@ -2178,7 +2246,7 @@ static ssize_t safe_pwrite64(Task* t, const void* buf, ssize_t buf_size,
   };
 
   if (mappings_to_fix.empty()) {
-    return pwrite64(t->vm()->mem_fd(), buf, buf_size, addr.as_int());
+    return pwrite_all_fallible(t->vm()->mem_fd(), buf, buf_size, addr.as_int());
   }
 
   AutoRemoteSyscalls remote(t);
@@ -2187,7 +2255,7 @@ static ssize_t safe_pwrite64(Task* t, const void* buf, ssize_t buf_size,
     remote.infallible_syscall(mprotect_syscallno, m.start(), m.size(),
                               m.prot() | PROT_WRITE);
   }
-  ssize_t nwritten = pwrite64(t->vm()->mem_fd(), buf, buf_size, addr.as_int());
+  ssize_t nwritten = pwrite_all_fallible(t->vm()->mem_fd(), buf, buf_size, addr.as_int());
   for (auto& m : mappings_to_fix) {
     remote.infallible_syscall(mprotect_syscallno, m.start(), m.size(),
                               m.prot());
@@ -2376,23 +2444,29 @@ static long perform_remote_clone(AutoRemoteSyscalls& remote,
   return child;
 }
 
-static void setup_fd_table(FdTable& fds, int tracee_socket_fd_number) {
-  fds.add_monitor(STDOUT_FILENO, new StdioMonitor(STDOUT_FILENO));
-  fds.add_monitor(STDERR_FILENO, new StdioMonitor(STDERR_FILENO));
-  fds.add_monitor(RR_MAGIC_SAVE_DATA_FD, new MagicSaveDataMonitor());
-  fds.add_monitor(RR_RESERVED_ROOT_DIR_FD, new PreserveFileMonitor());
-  fds.add_monitor(tracee_socket_fd_number, new PreserveFileMonitor());
+static void setup_fd_table(Task* t, FdTable& fds, int tracee_socket_fd_number) {
+  fds.add_monitor(t, STDOUT_FILENO, new StdioMonitor(STDOUT_FILENO));
+  fds.add_monitor(t, STDERR_FILENO, new StdioMonitor(STDERR_FILENO));
+  fds.add_monitor(t, RR_MAGIC_SAVE_DATA_FD, new MagicSaveDataMonitor());
+  fds.add_monitor(t, RR_RESERVED_ROOT_DIR_FD, new PreserveFileMonitor());
+  fds.add_monitor(t, tracee_socket_fd_number, new PreserveFileMonitor());
 }
 
-static void set_cpu_affinity(int cpu) {
+// Returns true if we succeeded, false if we failed because the
+// requested CPU does not exist/is not available.
+static bool set_cpu_affinity(int cpu) {
   DEBUG_ASSERT(cpu >= 0);
 
   cpu_set_t mask;
   CPU_ZERO(&mask);
   CPU_SET(cpu, &mask);
   if (0 > sched_setaffinity(0, sizeof(mask), &mask)) {
+    if (errno == EINVAL) {
+      return false;
+    }
     FATAL() << "Couldn't bind to CPU " << cpu;
   }
+  return true;
 }
 
 static void spawned_child_fatal_error(const ScopedFd& err_fd,
@@ -2519,11 +2593,7 @@ static SeccompFilter<struct sock_filter> create_seccomp_filter() {
  * things go wrong because we have no ptracer and the seccomp filter demands
  * one.
  */
-static void set_up_seccomp_filter(SeccompFilter<struct sock_filter>& f,
-                                  int err_fd) {
-  struct sock_fprog prog = { (unsigned short)f.filters.size(),
-                             f.filters.data() };
-
+static void set_up_seccomp_filter(const struct sock_fprog& prog, int err_fd) {
   /* Note: the filter is installed only for record. This call
    * will be emulated (not passed to the kernel) in the replay. */
   if (0 > prctl(PR_SET_SECCOMP, SECCOMP_MODE_FILTER, (uintptr_t)&prog, 0, 0)) {
@@ -2535,18 +2605,11 @@ static void set_up_seccomp_filter(SeccompFilter<struct sock_filter>& f,
 }
 
 static void run_initial_child(Session& session, const ScopedFd& error_fd,
-                              const ScopedFd& sock_fd,
-                              int sock_fd_number,
-                              const string& exe_path,
-                              const vector<string>& argv,
-                              const vector<string>& envp) {
-  // Move all allocations here, before we signal the ptracer and start
-  // checking system calls. We don't want stray system calls performed by
-  // the allocator to show up in the trace.
-  const char* exe_path_cstr = exe_path.c_str();
-  StringVectorToCharArray argv_array(argv);
-  StringVectorToCharArray envp_array(envp);
-  SeccompFilter<struct sock_filter> filter = create_seccomp_filter();
+                              const ScopedFd& sock_fd, int sock_fd_number,
+                              const char* exe_path_cstr,
+                              char* const argv_array[],
+                              char* const envp_array[],
+                              const struct sock_fprog& seccomp_prog) {
   pid_t pid = getpid();
 
   set_up_process(session, error_fd, sock_fd, sock_fd_number);
@@ -2558,7 +2621,7 @@ static void run_initial_child(Session& session, const ScopedFd& error_fd,
   ::kill(pid, SIGSTOP);
 
   // This code must run after rr has taken ptrace control.
-  set_up_seccomp_filter(filter, error_fd);
+  set_up_seccomp_filter(seccomp_prog, error_fd);
 
   // We do a small amount of dummy work here to retire
   // some branches in order to ensure that the ticks value is
@@ -2575,7 +2638,7 @@ static void run_initial_child(Session& session, const ScopedFd& error_fd,
 
   CPUIDBugDetector::run_detection_code();
 
-  execve(exe_path_cstr, argv_array.get(), envp_array.get());
+  execve(exe_path_cstr, argv_array, envp_array);
 
   switch (errno) {
     case ENOENT:
@@ -2594,7 +2657,7 @@ static void run_initial_child(Session& session, const ScopedFd& error_fd,
 /*static*/ Task* Task::spawn(Session& session, const ScopedFd& error_fd,
                              ScopedFd* sock_fd_out,
                              int* tracee_socket_fd_number_out,
-                             const TraceStream& trace,
+                             TraceStream& trace,
                              const std::string& exe_path,
                              const std::vector<std::string>& argv,
                              const std::vector<std::string>& envp,
@@ -2625,16 +2688,43 @@ static void run_initial_child(Session& session, const ScopedFd& error_fd,
   }
   *tracee_socket_fd_number_out = fd_number;
 
-  if (trace.bound_to_cpu() >= 0) {
+  int cpu_index = session.cpu_binding(trace);
+  if (cpu_index >= 0) {
     // Set CPU affinity now, after we've created any helper threads
     // (so they aren't affected), but before we create any
     // tracees (so they are all affected).
     // Note that we're binding rr itself to the same CPU as the
     // tracees, since this seems to help performance.
-    set_cpu_affinity(trace.bound_to_cpu());
+    if (!set_cpu_affinity(cpu_index)) {
+      if (session.has_cpuid_faulting() && !session.is_recording()) {
+        cpu_index = choose_cpu(BIND_CPU);
+        if (!set_cpu_affinity(cpu_index)) {
+          FATAL() << "Can't bind to requested CPU " << cpu_index
+                  << " even after we re-selected it";
+        }
+        LOG(warn) << "Bound to CPU " << cpu_index
+                  << "instead of selected " << trace.bound_to_cpu()
+                  << "because the latter is not available;\n"
+                  << "Hoping tracee doesn't use LSL instruction!";
+        trace.set_bound_cpu(cpu_index);
+      } else {
+        FATAL() << "Can't bind to requested CPU " << cpu_index
+                << ", and CPUID faulting not available";
+      }
+    }
   }
 
   pid_t tid;
+  // After fork() in a multithreaded program, the child can safely call only
+  // async-signal-safe functions, and malloc is not one of them (breaks e.g.
+  // with tcmalloc).
+  // Doing the allocations before the fork duplicates the allocations, but
+  // prevents errors.
+  StringVectorToCharArray argv_array(argv);
+  StringVectorToCharArray envp_array(envp);
+  SeccompFilter<struct sock_filter> filter = create_seccomp_filter();
+  struct sock_fprog prog = {(unsigned short)filter.filters.size(),
+                            filter.filters.data()};
   do {
     tid = fork();
     // fork() can fail with EAGAIN due to temporary load issues. In such
@@ -2642,8 +2732,8 @@ static void run_initial_child(Session& session, const ScopedFd& error_fd,
   } while (0 > tid && errno == EAGAIN);
 
   if (0 == tid) {
-    run_initial_child(session, error_fd, sock, fd_number, exe_path, argv,
-                      envp);
+    run_initial_child(session, error_fd, sock, fd_number, exe_path.c_str(),
+                      argv_array.get(), envp_array.get(), prog);
     // run_initial_child never returns
   }
 
@@ -2657,7 +2747,10 @@ static void run_initial_child(Session& session, const ScopedFd& error_fd,
   // parented by the init process, i.e. effectively leaked. After PTRACE_SEIZE
   // with PTRACE_O_EXITKILL, the tracee will die if rr dies.
   intptr_t options = PTRACE_O_TRACESYSGOOD | PTRACE_O_TRACEFORK |
-                     PTRACE_O_TRACECLONE | PTRACE_O_TRACEEXIT;
+                     PTRACE_O_TRACECLONE;
+  if (!Flags::get().disable_ptrace_exit_events) {
+    options |= PTRACE_O_TRACEEXIT;
+  }
   if (session.is_recording()) {
     options |= PTRACE_O_TRACEVFORK | PTRACE_O_TRACESECCOMP | PTRACE_O_TRACEEXEC;
   }
@@ -2693,7 +2786,7 @@ static void run_initial_child(Session& session, const ScopedFd& error_fd,
   auto as = session.create_vm(t);
   t->as.swap(as);
   t->fds = FdTable::create(t);
-  setup_fd_table(*t->fds, fd_number);
+  setup_fd_table(t, *t->fds, fd_number);
 
   // Install signal handler here, so that when creating the first RecordTask
   // it sees the exact same signal state in the parent as will be in the child.
