@@ -34,7 +34,6 @@ namespace rr {
 
 class AutoRemoteSyscalls;
 class RecordSession;
-class RecordTask;
 class ReplaySession;
 class ScopedFd;
 class Session;
@@ -177,6 +176,10 @@ public:
    * Stat |fd| in the context of this task's fd table.
    */
   struct stat stat_fd(int fd);
+  /**
+   * Lstat |fd| in the context of this task's fd table.
+   */
+  struct stat lstat_fd(int fd);
   /**
    * Open |fd| in the context of this task's fd table.
    */
@@ -236,6 +239,11 @@ public:
   /** Return the current $ip of this. */
   remote_code_ptr ip() { return regs().ip(); }
 
+  /**
+   * Emulate a jump to a new IP, updating the ticks counter as appropriate.
+   */
+  void emulate_jump(remote_code_ptr);
+
   /**
    * Return true if this is at an arm-desched-event or
    * disarm-desched-event syscall.
@@ -472,7 +480,7 @@ public:
   bool set_debug_regs(const DebugRegs& regs);
 
   uintptr_t get_debug_reg(size_t regno);
-  void set_debug_reg(size_t regno, uintptr_t value);
+  bool set_debug_reg(size_t regno, uintptr_t value);
 
   /** Update the thread area to |addr|. */
   void set_thread_area(remote_ptr<struct ::user_desc> tls);
@@ -613,10 +621,11 @@ public:
   void write_mem(remote_ptr<T> child_addr, const T* val) = delete;
 
   template <typename T>
-  void write_mem(remote_ptr<T> child_addr, const T* val, int count) {
+  void write_mem(remote_ptr<T> child_addr, const T* val, int count,
+                 bool* ok = nullptr) {
     DEBUG_ASSERT(type_has_no_holes<T>());
     write_bytes_helper(child_addr, sizeof(*val) * count,
-                       static_cast<const void*>(val));
+                       static_cast<const void*>(val), ok);
   }
 
   /**
@@ -658,11 +667,11 @@ public:
 
   /**
    * Open /proc/[tid]/mem fd for our AddressSpace, closing the old one
-   * first.
-   * This never fails. If necessary we force the tracee to open the file
+   * first. If necessary we force the tracee to open the file
    * itself and smuggle the fd back to us.
+   * Returns false if the process no longer exists.
    */
-  void open_mem_fd();
+  bool open_mem_fd();
 
   /**
    * Calls open_mem_fd if this task's AddressSpace doesn't already have one.
@@ -760,6 +769,7 @@ public:
                                  : scratch_ptr + scratch_size;
   }
   void setup_preload_thread_locals();
+  void setup_preload_thread_locals_from_clone(Task* origin);
   const ThreadLocals& fetch_preload_thread_locals();
   void activate_preload_thread_locals();
 
@@ -839,6 +849,11 @@ protected:
    */
   virtual void post_wait_clone(Task*, int) {}
 
+  /**
+   * Internal method called after the clone to fix up the new address space.
+   */
+  virtual bool post_vm_clone(CloneReason reason, int flags, Task* origin);
+
   template <typename Arch>
   void on_syscall_exit_arch(int syscallno, const Registers& regs);
 
@@ -955,7 +970,7 @@ protected:
    */
   static Task* spawn(Session& session, const ScopedFd& error_fd,
                      ScopedFd* sock_fd_out, int* tracee_socket_fd_number_out,
-                     const TraceStream& trace, const std::string& exe_path,
+                     TraceStream& trace, const std::string& exe_path,
                      const std::vector<std::string>& argv,
                      const std::vector<std::string>& envp, pid_t rec_tid = -1);
 
