@@ -7,8 +7,10 @@
 #include <inttypes.h>
 #include <limits.h>
 #include <sched.h>
+#include <sys/file.h>
 #include <sys/wait.h>
 #include <sysexits.h>
+#include <dirent.h>
 
 #include <algorithm>
 #include <fstream>
@@ -110,7 +112,7 @@ static string default_rr_trace_dir() {
   return cached_dir;
 }
 
-static string trace_save_dir() {
+string trace_save_dir() {
   const char* output_dir = getenv("_RR_TRACE_DIR");
   return output_dir ? output_dir : default_rr_trace_dir();
 }
@@ -119,11 +121,27 @@ static string latest_trace_symlink() {
   return trace_save_dir() + "/latest-trace";
 }
 
-/**
- * Create the default ~/.rr directory if it doesn't already exist.
- */
-static void ensure_default_rr_trace_dir() {
-  ensure_dir(default_rr_trace_dir(), "trace directory", S_IRWXU);
+string resolve_trace_name(const string& trace_name)
+{
+  if (trace_name.empty()) {
+    return latest_trace_symlink();
+  }
+
+  // Single-component paths are looked up first in the current directory, next
+  // in the default trace dir.
+
+  if (trace_name.find('/') == string::npos) {
+    if (dir_exists(trace_name)) {
+      return trace_name;
+    }
+
+    string resolved_trace_name = trace_save_dir() + "/" + trace_name;
+    if (dir_exists(resolved_trace_name)) {
+      return resolved_trace_name;
+    }
+  }
+
+  return trace_name;
 }
 
 class CompressedWriterOutputStream : public kj::OutputStream {
@@ -157,13 +175,13 @@ public:
       throw IOException();
     }
   }
-  virtual kj::ArrayPtr<const byte> tryGetReadBuffer() {
+  virtual kj::ArrayPtr<const capnp::byte> tryGetReadBuffer() {
     const uint8_t* p;
     size_t size;
     if (!reader.get_buffer(&p, &size)) {
       throw IOException();
     }
-    return kj::ArrayPtr<const byte>(p, size);
+    return kj::ArrayPtr<const capnp::byte>(p, size);
   }
 
 private:
@@ -171,7 +189,9 @@ private:
 };
 
 TraceStream::TraceStream(const string& trace_dir, FrameTime initial_time)
-    : trace_dir(real_path(trace_dir)), global_time(initial_time) {}
+    : trace_dir(real_path(trace_dir))
+    , global_time(initial_time)
+   {}
 
 string TraceStream::file_data_clone_file_name(const TaskUid& tuid) {
   stringstream ss;
@@ -203,12 +223,12 @@ bool TraceReader::good() const {
   return true;
 }
 
-static kj::ArrayPtr<const byte> str_to_data(const string& str) {
-  return kj::ArrayPtr<const byte>(reinterpret_cast<const byte*>(str.data()),
-                                  str.size());
+static kj::ArrayPtr<const capnp::byte> str_to_data(const string& str) {
+  return kj::ArrayPtr<const capnp::byte>(
+      reinterpret_cast<const capnp::byte*>(str.data()), str.size());
 }
 
-static string data_to_str(const kj::ArrayPtr<const byte>& data) {
+static string data_to_str(const kj::ArrayPtr<const capnp::byte>& data) {
   if (memchr(data.begin(), 0, data.size())) {
     FATAL() << "Invalid string: contains null character";
   }
@@ -325,6 +345,30 @@ static Event from_trace_signal(EventType type, trace::Signal::Reader signal) {
                            from_trace_disposition(signal.getDisposition())));
 }
 
+static trace::TicksSemantics to_trace_ticks_semantics(TicksSemantics semantics) {
+  switch (semantics) {
+    case TICKS_RETIRED_CONDITIONAL_BRANCHES:
+      return trace::TicksSemantics::RETIRED_CONDITIONAL_BRANCHES;
+    case TICKS_TAKEN_BRANCHES:
+      return trace::TicksSemantics::TAKEN_BRANCHES;
+    default:
+      FATAL() << "Unknown ticks semantics";
+      return trace::TicksSemantics::RETIRED_CONDITIONAL_BRANCHES;
+  }
+}
+
+static TicksSemantics from_trace_ticks_semantics(trace::TicksSemantics semantics) {
+  switch (semantics) {
+    case trace::TicksSemantics::RETIRED_CONDITIONAL_BRANCHES:
+      return TICKS_RETIRED_CONDITIONAL_BRANCHES;
+    case trace::TicksSemantics::TAKEN_BRANCHES:
+      return TICKS_TAKEN_BRANCHES;
+    default:
+      FATAL() << "Unknown ticks semantics";
+      return TICKS_RETIRED_CONDITIONAL_BRANCHES;
+  }
+}
+
 static pid_t i32_to_tid(int tid) {
   if (tid <= 0) {
     FATAL() << "Invalid tid";
@@ -437,6 +481,8 @@ void TraceWriter::write_frame(RecordTask* t, const Event& ev,
           auto opened = e.opened[i];
           o.setFd(opened.fd);
           o.setPath(str_to_data(opened.path));
+          o.setDevice(opened.device);
+          o.setInode(opened.inode);
         }
       }
       break;
@@ -451,7 +497,7 @@ void TraceWriter::write_frame(RecordTask* t, const Event& ev,
     CompressedWriterOutputStream stream(events);
     writePackedMessage(stream, frame_msg);
   } catch (...) {
-    FATAL() << "Unable to write tasks";
+    FATAL() << "Unable to write events";
   }
 
   tick_time();
@@ -481,7 +527,7 @@ TraceFrame TraceReader::read_frame() {
   if (ret.ticks_ < 0) {
     FATAL() << "Invalid ticks value";
   }
-  ret.monotonic_time_ = frame.getMonotonicSec();
+  monotonic_time_ = ret.monotonic_time_ = frame.getMonotonicSec();
 
   SupportedArch arch = from_trace_arch(frame.getArch());
   ret.recorded_regs.set_arch(arch);
@@ -573,8 +619,12 @@ TraceFrame TraceReader::read_frame() {
           auto open = data.getOpenedFds();
           syscall_ev.opened.resize(open.size());
           for (size_t i = 0; i < open.size(); ++i) {
-            syscall_ev.opened[i].fd = check_fd(open[i].getFd());
-            syscall_ev.opened[i].path = data_to_str(open[i].getPath());
+            auto& opened = syscall_ev.opened[i];
+            const auto& o = open[i];
+            opened.fd = check_fd(o.getFd());
+            opened.path = data_to_str(o.getPath());
+            opened.device = o.getDevice();
+            opened.inode = o.getInode();
           }
           break;
         }
@@ -790,7 +840,8 @@ static bool starts_with(const string& s, const string& with) {
 
 TraceWriter::RecordInTrace TraceWriter::write_mapped_region(
     RecordTask* t, const KernelMapping& km, const struct stat& stat,
-    MappingOrigin origin) {
+    const vector<TraceRemoteFd>& extra_fds, MappingOrigin origin,
+    bool skip_monitoring_mapped_fd) {
   MallocMessageBuilder map_msg;
   trace::MMap::Builder map = map_msg.initRoot<trace::MMap>();
   map.setFrameTime(global_time);
@@ -807,6 +858,14 @@ TraceWriter::RecordInTrace TraceWriter::write_mapped_region(
   map.setStatGid(stat.st_gid);
   map.setStatSize(stat.st_size);
   map.setStatMTime(stat.st_mtime);
+  auto fds = map.initExtraFds(extra_fds.size());
+  for (size_t i = 0; i < extra_fds.size(); ++i) {
+    auto e = fds[i];
+    auto& r = extra_fds[i];
+    e.setTid(r.tid);
+    e.setFd(r.fd);
+  }
+  map.setSkipMonitoringMappedFd(skip_monitoring_mapped_fd);
   auto src = map.getSource();
   string backing_file_name;
 
@@ -836,7 +895,14 @@ TraceWriter::RecordInTrace TraceWriter::write_mapped_region(
       // the entire file is copied, not just the used region, which is why we
       // don't do this for all files.
       // Don't bother trying to copy [vdso].
-      if ((km.prot() & PROT_EXEC) && copy_file(file_name, &backing_file_name)) {
+      // Don't try to copy files that use shared mappings. We do not want to
+      // create a shared mapping of a file stored in the trace. This means
+      // debuggers can't find the file, but the Linux loader doesn't create
+      // shared mappings so situations where a shared-mapped executable contains
+      // usable debug info should be very rare at best...
+      if ((km.prot() & PROT_EXEC) &&
+          copy_file(file_name, &backing_file_name) &&
+          !(km.flags() & MAP_SHARED)) {
         src.initFile().setBackingFileName(str_to_data(backing_file_name));
       } else {
         src.setTrace();
@@ -876,7 +942,8 @@ TraceWriter::RecordInTrace TraceWriter::write_mapped_region(
 }
 
 void TraceWriter::write_mapped_region_to_alternative_stream(
-    CompressedWriter& mmaps, const MappedData& data, const KernelMapping& km) {
+    CompressedWriter& mmaps, const MappedData& data, const KernelMapping& km,
+    const vector<TraceRemoteFd>& extra_fds, bool skip_monitoring_mapped_fd) {
   MallocMessageBuilder map_msg;
   trace::MMap::Builder map = map_msg.initRoot<trace::MMap>();
 
@@ -890,6 +957,14 @@ void TraceWriter::write_mapped_region_to_alternative_stream(
   map.setFlags(km.flags());
   map.setFileOffsetBytes(km.file_offset_bytes());
   map.setStatSize(data.file_size_bytes);
+  auto fds = map.initExtraFds(extra_fds.size());
+  for (size_t i = 0; i < extra_fds.size(); ++i) {
+    auto e = fds[i];
+    auto& r = extra_fds[i];
+    e.setTid(r.tid);
+    e.setFd(r.fd);
+  }
+  map.setSkipMonitoringMappedFd(skip_monitoring_mapped_fd);
   auto src = map.getSource();
   switch (data.source) {
     case TraceReader::SOURCE_ZERO:
@@ -916,7 +991,9 @@ void TraceWriter::write_mapped_region_to_alternative_stream(
 
 KernelMapping TraceReader::read_mapped_region(MappedData* data, bool* found,
                                               ValidateSourceFile validate,
-                                              TimeConstraint time_constraint) {
+                                              TimeConstraint time_constraint,
+                                              vector<TraceRemoteFd>* extra_fds,
+                                              bool* skip_monitoring_mapped_fd) {
   if (found) {
     *found = false;
   }
@@ -947,6 +1024,16 @@ KernelMapping TraceReader::read_mapped_region(MappedData* data, bool* found,
     }
     data->data_offset_bytes = 0;
     data->file_size_bytes = map.getStatSize();
+    if (extra_fds) {
+      const auto& fds = map.getExtraFds();
+      for (size_t i = 0; i < fds.size(); ++i) {
+        const auto& f = fds[i];
+        extra_fds->push_back({ f.getTid(), f.getFd() });
+      }
+    }
+    if (skip_monitoring_mapped_fd) {
+      *skip_monitoring_mapped_fd = map.getSkipMonitoringMappedFd();
+    }
     auto src = map.getSource();
     switch (src.which()) {
       case trace::MMap::Source::Which::ZERO:
@@ -1054,52 +1141,60 @@ bool TraceReader::read_raw_data_metadata_for_frame(RawDataMetadata& d) {
   return true;
 }
 
-void TraceWriter::close() {
-  for (auto& w : writers) {
-    w->close();
+static string make_trace_dir(const string& exe_path, const string& output_trace_dir) {
+  if (!output_trace_dir.empty()) {
+    // save trace dir in given output trace dir with option -o
+    int ret = mkdir(output_trace_dir.c_str(), S_IRWXU | S_IRWXG);
+    if (ret == 0) {
+      return output_trace_dir;
+    }
+    if (EEXIST == errno) {
+      // directory already exists
+      FATAL() << "Directory `" << output_trace_dir << "' already exists.";
+    } else {
+      FATAL() << "Unable to create trace directory `" << output_trace_dir << "'";
+    }
+  } else {
+    // save trace dir set in _RR_TRACE_DIR or in the default trace dir
+    ensure_dir(trace_save_dir(), "trace directory", S_IRWXU);
+
+    // Find a unique trace directory name.
+    int nonce = 0;
+    int ret;
+    string dir;
+    do {
+      stringstream ss;
+      ss << trace_save_dir() << "/" << basename(exe_path.c_str()) << "-"
+         << nonce++;
+      dir = ss.str();
+      ret = mkdir(dir.c_str(), S_IRWXU | S_IRWXG);
+    } while (ret && EEXIST == errno);
+
+    if (ret) {
+      FATAL() << "Unable to create trace directory `" << dir << "'";
+    }
+
+    return dir;
   }
 
-  string incomplete_path = incomplete_version_path();
-  string path = version_path();
-  if (rename(incomplete_path.c_str(), path.c_str()) < 0) {
-    FATAL() << "Unable to create version file " << path;
-  }
-}
-
-static string make_trace_dir(const string& exe_path) {
-  ensure_default_rr_trace_dir();
-
-  // Find a unique trace directory name.
-  int nonce = 0;
-  int ret;
-  string dir;
-  do {
-    stringstream ss;
-    ss << trace_save_dir() << "/" << basename(exe_path.c_str()) << "-"
-       << nonce++;
-    dir = ss.str();
-    ret = mkdir(dir.c_str(), S_IRWXU | S_IRWXG);
-  } while (ret && EEXIST == errno);
-
-  if (ret) {
-    FATAL() << "Unable to create trace directory `" << dir << "'";
-  }
-
-  return dir;
+  return ""; // not reached
 }
 
 #define STR_HELPER(x) #x
 #define STR(x) STR_HELPER(x)
 
-TraceWriter::TraceWriter(const std::string& file_name, int bind_to_cpu,
-                         bool has_cpuid_faulting,
-                         const DisableCPUIDFeatures& disable_cpuid_features)
-    : TraceStream(make_trace_dir(file_name),
+TraceWriter::TraceWriter(const std::string& file_name,
+                         int bind_to_cpu,
+                         const string& output_trace_dir,
+                         TicksSemantics ticks_semantics_)
+    : TraceStream(make_trace_dir(file_name, output_trace_dir),
                   // Somewhat arbitrarily start the
                   // global time from 1.
                   1),
+      ticks_semantics_(ticks_semantics_),
       mmap_count(0),
-      supports_file_data_cloning_(false) {
+      has_cpuid_faulting_(false) {
+  this->ticks_semantics_ = ticks_semantics_;
   this->bind_to_cpu = bind_to_cpu;
 
   for (Substream s = SUBSTREAM_FIRST; s < SUBSTREAM_COUNT; ++s) {
@@ -1108,39 +1203,18 @@ TraceWriter::TraceWriter(const std::string& file_name, int bind_to_cpu,
   }
 
   string ver_path = incomplete_version_path();
-  ScopedFd version_fd(ver_path.c_str(), O_RDWR | O_CREAT, 0600);
+  version_fd = ScopedFd(ver_path.c_str(), O_RDWR | O_CREAT | O_CLOEXEC, 0600);
   if (!version_fd.is_open()) {
     FATAL() << "Unable to create " << ver_path;
   }
+  // Take an exclusive lock and hold it until we rename the file at
+  // the end of recording and then close our file descriptor.
+  if (flock(version_fd, LOCK_EX | LOCK_NB) != 0) {
+    FATAL() << "Unable to lock " << ver_path;
+  }
   static const char buf[] = STR(TRACE_VERSION) "\n";
-  if (write(version_fd, buf, sizeof(buf) - 1) != (ssize_t)sizeof(buf) - 1) {
-    FATAL() << "Unable to write " << ver_path;
-  }
-
-  // We are now bound to the selected CPU (if any), so collect CPUID records
-  // (which depend on the bound CPU number).
-  vector<CPUIDRecord> cpuid_records = all_cpuid_records();
-  for (auto& r : cpuid_records) {
-    disable_cpuid_features.amend_cpuid_data(r.eax_in, r.ecx_in, &r.out);
-  }
-
-  MallocMessageBuilder header_msg;
-  trace::Header::Builder header = header_msg.initRoot<trace::Header>();
-  header.setBindToCpu(bind_to_cpu);
-  header.setHasCpuidFaulting(has_cpuid_faulting);
-  header.setCpuidRecords(
-      Data::Reader(reinterpret_cast<const uint8_t*>(cpuid_records.data()),
-                   cpuid_records.size() * sizeof(CPUIDRecord)));
-  header.setXcr0(xcr0());
-  header.setSyscallbufProtocolVersion(SYSCALLBUF_PROTOCOL_VERSION);
-  // Add a random UUID to the trace metadata. This lets tools identify a trace
-  // easily.
-  uint8_t uuid[16];
-  good_random(uuid, sizeof(uuid));
-  header.setUuid(Data::Reader(uuid, sizeof(uuid)));
-  try {
-    writePackedMessageToFd(version_fd, header_msg);
-  } catch (...) {
+  size_t buf_len = sizeof(buf) - 1;
+  if (write(version_fd, buf, buf_len) != (ssize_t)buf_len) {
     FATAL() << "Unable to write " << ver_path;
   }
 
@@ -1154,11 +1228,7 @@ TraceWriter::TraceWriter(const std::string& file_name, int bind_to_cpu,
   btrfs_ioctl_clone_range_args clone_args;
   clone_args.src_fd = version_fd;
   clone_args.src_offset = 0;
-  off_t offset = lseek(version_fd, 0, SEEK_END);
-  if (offset <= 0) {
-    FATAL() << "Unable to lseek " << ver_path;
-  }
-  clone_args.src_length = offset;
+  clone_args.src_length = buf_len;
   clone_args.dest_offset = 0;
   if (ioctl(version_clone_fd, BTRFS_IOC_CLONE_RANGE, &clone_args) == 0) {
     supports_file_data_cloning_ = true;
@@ -1171,6 +1241,63 @@ TraceWriter::TraceWriter(const std::string& file_name, int bind_to_cpu,
   }
 }
 
+void TraceWriter::setup_cpuid_records(bool has_cpuid_faulting,
+                                      const DisableCPUIDFeatures& disable_cpuid_features) {
+  has_cpuid_faulting_ = has_cpuid_faulting;
+  // We are now bound to the selected CPU (if any), so collect CPUID records
+  // (which depend on the bound CPU number).
+  cpuid_records = all_cpuid_records();
+  // Modify the recorded cpuid data only if cpuid faulting is available. If it
+  // is not available, the tracee will see unmodified data and should also see
+  // that in handle_unrecorded_cpuid_fault (which is sourced from this data).
+  if (has_cpuid_faulting) {
+    for (auto& r : cpuid_records) {
+      disable_cpuid_features.amend_cpuid_data(r.eax_in, r.ecx_in, &r.out);
+    }
+  }
+}
+
+void TraceWriter::close(CloseStatus status, const TraceUuid* uuid) {
+  for (auto& w : writers) {
+    w->close();
+  }
+
+  MallocMessageBuilder header_msg;
+  trace::Header::Builder header = header_msg.initRoot<trace::Header>();
+  header.setBindToCpu(this->bind_to_cpu);
+  header.setHasCpuidFaulting(has_cpuid_faulting_);
+  header.setCpuidRecords(
+      Data::Reader(reinterpret_cast<const uint8_t*>(cpuid_records.data()),
+                   cpuid_records.size() * sizeof(CPUIDRecord)));
+  header.setXcr0(xcr0());
+  header.setTicksSemantics(
+    to_trace_ticks_semantics(PerfCounters::default_ticks_semantics()));
+  header.setSyscallbufProtocolVersion(SYSCALLBUF_PROTOCOL_VERSION);
+  header.setPreloadThreadLocalsRecorded(true);
+  // Add a random UUID to the trace metadata. This lets tools identify a trace
+  // easily.
+  if (!uuid) {
+    uint8_t uuid[sizeof(TraceUuid::bytes)];
+    good_random(uuid, sizeof(uuid));
+    header.setUuid(Data::Reader(uuid, sizeof(uuid)));
+  } else {
+    header.setUuid(Data::Reader(uuid->bytes, sizeof(TraceUuid::bytes)));
+  }
+  header.setOk(status == CLOSE_OK);
+  try {
+    writePackedMessageToFd(version_fd, header_msg);
+  } catch (...) {
+    FATAL() << "Unable to write " << incomplete_version_path();
+  }
+
+  string incomplete_path = incomplete_version_path();
+  string path = version_path();
+  if (rename(incomplete_path.c_str(), path.c_str()) < 0) {
+    FATAL() << "Unable to create version file " << path;
+  }
+  version_fd.close();
+}
+
 void TraceWriter::make_latest_trace() {
   string link_name = latest_trace_symlink();
   // Try to update the symlink to |this|.  We only try attempt
@@ -1215,7 +1342,7 @@ void TraceReader::rewind() {
 }
 
 TraceReader::TraceReader(const string& dir)
-    : TraceStream(dir.empty() ? latest_trace_symlink() : dir, 1) {
+    : TraceStream(resolve_trace_name(dir), 1) {
   for (Substream s = SUBSTREAM_FIRST; s < SUBSTREAM_COUNT; ++s) {
     readers[s] = unique_ptr<CompressedReader>(new CompressedReader(path(s)));
   }
@@ -1286,11 +1413,21 @@ TraceReader::TraceReader(const string& dir)
   trace_uses_cpuid_faulting = header.getHasCpuidFaulting();
   Data::Reader cpuid_records_bytes = header.getCpuidRecords();
   size_t len = cpuid_records_bytes.size() / sizeof(CPUIDRecord);
-  DEBUG_ASSERT(cpuid_records_bytes.size() == len * sizeof(CPUIDRecord));
+  if (cpuid_records_bytes.size() != len * sizeof(CPUIDRecord)) {
+    FATAL() << "Invalid CPUID records length";
+  }
   cpuid_records_.resize(len);
   memcpy(cpuid_records_.data(), cpuid_records_bytes.begin(),
          len * sizeof(CPUIDRecord));
   xcr0_ = header.getXcr0();
+  preload_thread_locals_recorded_ = header.getPreloadThreadLocalsRecorded();
+  ticks_semantics_ = from_trace_ticks_semantics(header.getTicksSemantics());
+  Data::Reader uuid = header.getUuid();
+  uuid_ = unique_ptr<TraceUuid>(new TraceUuid());
+  if (uuid.size() != sizeof(uuid_->bytes)) {
+    FATAL() << "Invalid UUID length";
+  }
+  memcpy(uuid_->bytes, uuid.begin(), sizeof(uuid_->bytes));
 
   // Set the global time at 0, so that when we tick it for the first
   // event, it matches the initial global time at recording, 1.
@@ -1314,6 +1451,7 @@ TraceReader::TraceReader(const TraceReader& other)
   cpuid_records_ = other.cpuid_records_;
   raw_recs = other.raw_recs;
   xcr0_ = other.xcr0_;
+  preload_thread_locals_recorded_ = other.preload_thread_locals_recorded_;
 }
 
 TraceReader::~TraceReader() {}
