@@ -1,4 +1,4 @@
-import StringIO
+from io import StringIO
 import sys
 
 class RawBytes(object):
@@ -178,7 +178,7 @@ def byte_array_name(name):
     return '%s_bytes' % name
 
 def generate_match_method(byte_array, template):
-    s = StringIO.StringIO()
+    s = StringIO()
     fields = template.fields()
     field_types = [f.c_type() for f in fields]
     field_names = [f.name for f in fields]
@@ -201,7 +201,7 @@ def generate_match_method(byte_array, template):
     return s.getvalue()
 
 def generate_substitute_method(byte_array, template):
-    s = StringIO.StringIO()
+    s = StringIO()
     fields = template.fields()
     field_types = [f.c_type() for f in fields]
     field_names = [f.name for f in fields]
@@ -223,7 +223,7 @@ def generate_substitute_method(byte_array, template):
     return s.getvalue()
 
 def generate_field_end_methods(byte_array, template):
-    s = StringIO.StringIO()
+    s = StringIO()
     offset = 0
     for chunk in template.chunks:
         offset += len(chunk)
@@ -232,20 +232,20 @@ def generate_field_end_methods(byte_array, template):
     return s.getvalue()
 
 def generate_size_member(byte_array):
-    s = StringIO.StringIO()
+    s = StringIO()
     s.write('  static const size_t size = sizeof(%s);' % byte_array)
     return s.getvalue()
 
 def generate(f):
     # Raw bytes.
-    for name, template in templates.iteritems():
+    for name, template in templates.items():
         bytes = template.bytes()
         f.write('static const uint8_t %s[] = { %s };\n'
                 % (byte_array_name(name), ', '.join(['0x%x' % b for b in bytes])))
     f.write('\n')
 
     # Objects representing assembly templates.
-    for name, template in templates.iteritems():
+    for name, template in templates.items():
         byte_array = byte_array_name(name)
         f.write("""class %(class_name)s {
 public:
