@@ -8,6 +8,7 @@
 #include <linux/seccomp.h>
 #include <linux/usbdevice_fs.h>
 #include <signal.h>
+#include <stdint.h>
 #include <sys/ioctl.h>
 #include <sys/ptrace.h>
 
@@ -81,6 +82,13 @@ namespace rr {
 #define SYS_SECCOMP 1
 #endif
 
+#ifndef PR_GET_SPECULATION_CTRL
+#define PR_GET_SPECULATION_CTRL 52
+#endif
+#ifndef PR_SET_SPECULATION_CTRL
+#define PR_SET_SPECULATION_CTRL 53
+#endif
+
 // These are defined by the include/linux/errno.h in the kernel tree.
 // Since userspace doesn't see these errnos in normal operation, that
 // header apparently isn't distributed with libc.
@@ -164,6 +172,9 @@ struct usbdevfs_streams {
 #ifndef TIOCGEXCL
 #define TIOCGEXCL _IOR('T', 0x40, int)
 #endif
+#ifndef TIOCGPTPEER
+#define TIOCGPTPEER _IO('T', 0x41)
+#endif
 
 struct rr_input_mask {
   uint32_t type;
@@ -303,6 +314,20 @@ struct rr_input_mask {
 #define ARCH_SET_CPUID 0x1012
 #endif
 
+// New in the 4.15 kernel
+#ifndef MAP_SYNC
+#define MAP_SYNC  0x80000
+#endif
+
+enum {
+  BPF_MAP_CREATE,
+  BPF_MAP_LOOKUP_ELEM,
+  BPF_MAP_UPDATE_ELEM,
+  BPF_MAP_DELETE_ELEM,
+  BPF_MAP_GET_NEXT_KEY,
+  BPF_PROG_LOAD,
+};
+
 } // namespace rr
 
 #endif /* RR_KERNEL_SUPPLEMENT_H_ */
