@@ -95,12 +95,6 @@ static inline const char* extract_file_name(const char* s) {
 #define SYSCALLBUF_LIB_FILENAME_PADDED SYSCALLBUF_LIB_FILENAME_BASE ".so:::"
 #define SYSCALLBUF_LIB_FILENAME_32 SYSCALLBUF_LIB_FILENAME_BASE "_32.so"
 
-/* This is pretty arbitrary. On Linux SIGPWR is sent to PID 1 (init) on
- * power failure, and it's unlikely rr will be recording that.
- * Note that SIGUNUSED means SIGSYS which actually *is* used (by seccomp),
- * so we can't use it. */
-#define SYSCALLBUF_DESCHED_SIGNAL SIGPWR
-
 /* Set this env var to enable syscall buffering. */
 #define SYSCALLBUF_ENABLED_ENV_VAR "_RR_USE_SYSCALLBUF"
 
@@ -260,8 +254,8 @@ struct preload_globals {
   /* 1 if chaos mode is enabled. DO NOT READ from rr during replay, because
      this field is not initialized in old traces. */
   unsigned char in_chaos;
-  /* Padding, currently unused; can be used later. */
-  unsigned char padding;
+  /* The signal to use for desched events */
+  unsigned char desched_sig;
   /* Number of cores to pretend we have. 0 means 1. rr sets this when
    * the preload library is initialized. */
   int pretend_num_cores;
@@ -272,6 +266,8 @@ struct preload_globals {
    * The rr supervisor modifies this array directly to dynamically turn
    * syscallbuf on and off for particular fds. fds outside the array range must
    * never use the syscallbuf.
+   * The last entry is set if *any* fd >= SYSCALLBUF_FDS_DISABLED_SIZE - 1
+   * has had buffering disabled.
    */
   VOLATILE char syscallbuf_fds_disabled[SYSCALLBUF_FDS_DISABLED_SIZE];
   /* mprotect records. Set by preload. */
@@ -384,7 +380,7 @@ struct preload_thread_locals {
   int cloned_file_data_fd;
   off_t cloned_file_data_offset;
   PTR(void) scratch_buf;
-  size_t scratch_size;
+  size_t usable_scratch_size;
 
   PTR(struct msghdr) notify_control_msg;
 };
@@ -435,7 +431,7 @@ struct rrcall_init_buffers_params {
   /* Returned pointer to rr's syscall scratch buffer */
   PTR(void) scratch_buf;
   uint32_t syscallbuf_size;
-  uint32_t scratch_size;
+  uint32_t usable_scratch_size;
 };
 
 /**
@@ -525,7 +521,7 @@ struct syscallbuf_hdr {
    */
   volatile uint8_t in_sigprocmask_critical_section;
   /* Nonzero when the syscall was aborted during preparation without doing
-   * anything */
+   * anything. This is set when a user seccomp filter forces a SIGSYS. */
   volatile uint8_t failed_during_preparation;
 
   struct syscallbuf_record recs[0];
@@ -579,7 +575,8 @@ inline static long stored_record_size(size_t length) {
 inline static int is_blacklisted_filename(const char* filename) {
   const char* f;
   if (strprefix("/dev/dri/", filename) || streq("/dev/nvidiactl", filename) ||
-      streq("/usr/share/alsa/alsa.conf", filename)) {
+      streq("/usr/share/alsa/alsa.conf", filename) ||
+      streq("/dev/nvidia-uvm", filename)) {
     return 1;
   }
   f = extract_file_name(filename);
@@ -630,13 +627,13 @@ inline static int is_proc_fd_dir(const char* filename) {
 /**
  * Returns nonzero if an attempted open() of |filename| can be syscall-buffered.
  * When this returns zero, the open must be forwarded to the rr process.
- * This is imperfect because it doesn't handle symbolic links, hard links,
- * files accessed with non-absolute paths, /proc mounted in differnet places,
- * etc etc etc. Handling those efficiently (no additional syscalls in
- * common cases) is a problem. Maybe we could afford fstat after every open...
+ * |filename| must be absolute.
+ * This is imperfect because it doesn't handle hard links and files (re)mounted
+ * in different places.
  */
 inline static int allow_buffered_open(const char* filename) {
-  return !is_blacklisted_filename(filename) && !is_gcrypt_deny_file(filename) &&
+  return filename &&
+         !is_blacklisted_filename(filename) && !is_gcrypt_deny_file(filename) &&
          !is_terminal(filename) && !is_proc_mem_file(filename) &&
          !is_proc_fd_dir(filename);
 }
