@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; c-basic-offset: 2; indent-tabs-mode: nil; -*- */
 
-#include "rrutil.h"
+#include "util.h"
 
 static int create_segment(size_t num_bytes) {
   char filename[] = "/dev/shm/rr-test-XXXXXX";
