@@ -28,7 +28,7 @@ int main(int argc, char** argv) {
   /* Test that checksumming doesn't care if we have a mmap
    * that is not backed by a sufficiently long file */
   size_t page_size = sysconf(_SC_PAGESIZE);
-  static const char name[] = "temp";
+  static const char name[] = "temp1";
   int fd = open(name, O_CREAT | O_RDWR | O_EXCL, 0600);
   /* Have it extend a couple of bytes into the second page */
   test_assert(0 == ftruncate(fd, page_size + 200));
@@ -38,7 +38,7 @@ int main(int argc, char** argv) {
   /* Make the second and third page PROT_NONE */
   test_assert(0 == mprotect(map_addr + page_size, 2 * page_size, PROT_NONE));
   /* Just some syscall to get some additional checksums in */
-  (void)geteuid();
+  event_syscall();
   test_assert(0 == unlink(name));
 
   /* The same again, but this time unmap the two pages in the middle */
@@ -53,14 +53,13 @@ int main(int argc, char** argv) {
   test_assert(0 == unlink(name2));
 
   /* Now map two temporary files right next to each other to make sure they're
-     not getting coralesced, causing trouble for the checksumming */
+     not getting coalesced, causing trouble for the checksumming */
   static const char name3[] = "temp3";
   fd = open(name3, O_CREAT | O_RDWR | O_EXCL, 0600);
   static const char name4[] = "temp4";
   int fd2 = open(name4, O_CREAT | O_RDWR | O_EXCL, 0600);
   ftruncate(fd, page_size);
   ftruncate(fd2, page_size);
-
   // Alloc a 2 page region first, then overwrite it.
   map_addr =
       mmap(NULL, 2 * page_size, PROT_NONE, MAP_PRIVATE | MAP_ANONYMOUS, 0, 0);
@@ -69,10 +68,21 @@ int main(int argc, char** argv) {
                                  MAP_PRIVATE | MAP_FIXED, fd, 0));
   test_assert(MAP_FAILED != mmap(map_addr + page_size, page_size, PROT_NONE,
                                  MAP_PRIVATE | MAP_FIXED, fd2, 0));
-  (void)geteuid();
+  event_syscall();
   test_assert(0 == unlink(name3));
   test_assert(0 == unlink(name4));
 
+  /* Check mapping of file that's (probably) not in a tmpfs, and make sure
+     we can coalesce mappings that are/are not beyond the end of the file. */
+  fd = open("/bin/ls", O_RDONLY);
+  long end = lseek(fd, 0, SEEK_END);
+  test_assert(end > 0);
+  map_addr = mmap(NULL, ceil_page_size(end) + page_size, PROT_READ, MAP_PRIVATE, fd, 0);
+  test_assert(MAP_FAILED != map_addr);
+  test_assert(MAP_FAILED != mmap(map_addr, page_size, PROT_READ,
+                                 MAP_PRIVATE | MAP_FIXED, fd, 0));
+  event_syscall();
+
   atomic_puts("EXIT-SUCCESS");
   return 0;
 }
