@@ -1,7 +1,7 @@
 /* -*- Mode: C; tab-width: 8; c-basic-offset: 2; indent-tabs-mode: nil; -*- */
 
 #include "nsutils.h"
-#include "rrutil.h"
+#include "util.h"
 
 int main(void) {
   pid_t pid;
@@ -12,7 +12,9 @@ int main(void) {
   }
 
   // This is the first child, therefore PID 1 in it's PID namespace
-  if ((pid = fork()) == 0) {
+  pid = fork();
+  test_assert(pid >= 0);
+  if (pid == 0) {
     // We will check that rr ps includes `(1)` here
     return 78;
   }
