@@ -1,10 +1,12 @@
-from rrutil import *
+from util import *
 
 send_gdb('handle SIGKILL stop')
 
 # Test that invalid syntax doesn't crash rr
 send_gdb('run run 100')
+expect_gdb('from the beginning')
 send_gdb('y')
+expect_gdb('(rr)')
 send_gdb('break main')
 expect_gdb('Breakpoint 1')
 send_gdb('c')
@@ -12,6 +14,7 @@ expect_gdb('Breakpoint 1')
 
 # Running with a big event number should reach the end of the recording
 send_gdb('run 10000')
+expect_gdb('from the beginning')
 send_gdb('y')
 expect_gdb('SIGKILL')
 send_gdb('c')
