@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; c-basic-offset: 2; indent-tabs-mode: nil; -*- */
 
-#include "rrutil.h"
+#include "util.h"
 
 int main(void) {
   pid_t child;
@@ -10,7 +10,7 @@ int main(void) {
   if (0 == (child = fork())) {
     nanosleep(&ts, NULL);
     kill(getpid(), SIGSTOP);
-    assert(0 && "child should not resume");
+    test_assert(0 && "child should not resume");
     return 77;
   }
 
