@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; c-basic-offset: 2; indent-tabs-mode: nil; -*- */
 
-#include "rrutil.h"
+#include "util.h"
 
 static void waittermsig(int sig, const char* waiter) {
   struct timespec ts = {.tv_sec = 1 };
