@@ -1,12 +1,12 @@
 /* -*- Mode: C; tab-width: 8; c-basic-offset: 2; indent-tabs-mode: nil; -*- */
 
-#include "rrutil.h"
+#include "util.h"
 
 static char data[10] = "0123456789";
 
 static void test(int use_pwritev) {
-  char name[] = "/tmp/rr-readv-XXXXXX";
-  int fd = mkstemp(name);
+  static const char name[] = "temp";
+  int fd = open(name, O_CREAT | O_EXCL | O_RDWR, 0600);
   struct {
     char ch[50];
   } * buf;
