@@ -1,8 +1,5 @@
 /* -*- Mode: C++; tab-width: 8; c-basic-offset: 2; indent-tabs-mode: nil; -*- */
 
-//#define FIRST_INTERESTING_EVENT 10700
-//#define LAST_INTERESTING_EVENT 10900
-
 #include "util.h"
 
 #include <arpa/inet.h>
@@ -49,6 +46,67 @@ using namespace std;
 
 namespace rr {
 
+template <typename Arch> static remote_ptr<typename Arch::unsigned_word> env_ptr(Task* t) {
+  auto stack_ptr = t->regs().sp().cast<typename Arch::unsigned_word>();
+
+  auto argc = t->read_mem(stack_ptr);
+  stack_ptr += argc + 1;
+
+  // Check final NULL in argv
+  auto null_ptr = t->read_mem(stack_ptr);
+  ASSERT(t, null_ptr == 0);
+  stack_ptr++;
+  return stack_ptr;
+}
+
+template <typename Arch> static vector<uint8_t> read_auxv_arch(Task* t) {
+  auto stack_ptr = env_ptr<Arch>(t);
+
+  // Should now point to envp
+  while (0 != t->read_mem(stack_ptr)) {
+    stack_ptr++;
+  }
+  stack_ptr++;
+  // should now point to ELF Auxiliary Table
+
+  vector<uint8_t> result;
+  while (true) {
+    auto pair_vec = t->read_mem(stack_ptr, 2);
+    stack_ptr += 2;
+    typename Arch::unsigned_word pair[2] = { pair_vec[0], pair_vec[1] };
+    result.resize(result.size() + sizeof(pair));
+    memcpy(result.data() + result.size() - sizeof(pair), pair, sizeof(pair));
+    if (pair[0] == 0) {
+      break;
+    }
+  }
+  return result;
+}
+
+vector<uint8_t> read_auxv(Task* t) {
+  RR_ARCH_FUNCTION(read_auxv_arch, t->arch(), t);
+}
+
+template <typename Arch> static vector<string> read_env_arch(Task* t) {
+  auto stack_ptr = env_ptr<Arch>(t);
+
+  // Should now point to envp
+  vector<string> result;
+  while (true) {
+    auto p = t->read_mem(stack_ptr);
+    stack_ptr++;
+    if (!p) {
+      break;
+    }
+    result.push_back(t->read_c_str(remote_ptr<char>(p)));
+  }
+  return result;
+}
+
+vector<string> read_env(Task* t) {
+  RR_ARCH_FUNCTION(read_env_arch, t->arch(), t);
+}
+
 // FIXME this function assumes that there's only one address space.
 // Should instead only look at the address space of the task in
 // question.
@@ -283,6 +341,12 @@ static bool is_task_buffer(const AddressSpace& as,
   return false;
 }
 
+struct ParsedChecksumLine {
+  remote_ptr<void> start;
+  remote_ptr<void> end;
+  unsigned int checksum;
+};
+
 /**
  * Either create and store checksums for each segment mapped in |t|'s
  * address space, or validate an existing computed checksum.  Behavior
@@ -312,66 +376,66 @@ static void iterate_checksums(Task* t, ChecksumMode mode,
     t->write_mem(in_replay_flag, (unsigned char)0);
   }
 
-  const AddressSpace& as = *t->vm();
+  AddressSpace& as = *t->vm();
+  vector<ParsedChecksumLine> checksums;
+  if (VALIDATE_CHECKSUMS == mode) {
+    while (true) {
+      char line[1024];
+      if (!fgets(line, sizeof(line), c.checksums_file)) {
+        break;
+      }
+      ParsedChecksumLine parsed;
+      unsigned long rec_start;
+      unsigned long rec_end;
+      int nparsed =
+          sscanf(line, "(%x) %lx-%lx", &parsed.checksum, &rec_start, &rec_end);
+      parsed.start = rec_start;
+      parsed.end = rec_end;
+      ASSERT(t, 3 == nparsed) << "Parsed " << nparsed << " items";
+      checksums.push_back(parsed);
+
+      as.ensure_replay_matches_single_recorded_mapping(t, MemoryRange(parsed.start, parsed.end));
+    }
+  }
+
+  auto checksum_iter = checksums.begin();
   for (auto it = as.maps().begin(); it != as.maps().end(); ++it) {
-    auto m = *it;
+    AddressSpace::Mapping m = *it;
     string raw_map_line = m.map.str();
     uint32_t rec_checksum = 0;
 
     if (VALIDATE_CHECKSUMS == mode) {
-      char line[1024];
-      if (!fgets(line, sizeof(line), c.checksums_file)) {
-        FATAL() << "Can't read checksum file";
-      }
-      unsigned long rec_start;
-      unsigned long rec_end;
-      unsigned tmp_checksum;
-      int nparsed =
-          sscanf(line, "(%x) %lx-%lx", &tmp_checksum, &rec_start, &rec_end);
-      rec_checksum = tmp_checksum;
-      remote_ptr<void> rec_start_addr = rec_start;
-      remote_ptr<void> rec_end_addr = rec_end;
-      ASSERT(t, 3 == nparsed) << "Parsed " << nparsed << " items";
-      for (; m.map.start() != rec_start_addr; m = *(++it)) {
+      ParsedChecksumLine parsed = *checksum_iter;
+      ++checksum_iter;
+      for (; m.map.start() != parsed.start; m = *(++it)) {
         if (is_task_buffer(as, m)) {
           // This region corresponds to a task scratch or syscall buffer. We
           // tear these down a little later during replay so just skip it for
           // now.
           continue;
         }
-        if (m.flags & AddressSpace::Mapping::IS_SIGBUS_REGION) {
-          // If we have artifical SIGBUS regions, those may (if the entire
-          // region was SIGBUS), but need not, have existed during recording.
-          continue;
-        }
-        FATAL() << "Segment " << rec_start_addr << "-" << rec_end_addr
+        FATAL() << "Segment " << parsed.start << "-" << parsed.end
                 << " changed to " << m.map << "??";
       }
-      // If the backing file is too short, we cut mappings short, to make sure
-      // have the same behavior as during recording. Tolerate this.
-      ASSERT(t, m.map.end() <= rec_end_addr)
-          << "Segment " << rec_start_addr << "-" << rec_end_addr
+      ASSERT(t, m.map.end() == parsed.end)
+          << "Segment " << parsed.start << "-" << parsed.end
           << " changed to " << m.map << "??";
-      if (is_start_of_scratch_region(t, rec_start_addr)) {
+      if (is_start_of_scratch_region(t, parsed.start)) {
         /* Replay doesn't touch scratch regions, so
          * their contents are allowed to diverge.
          * Tracees can't observe those segments unless
          * they do something sneaky (or disastrously
          * buggy). */
-        LOG(debug) << "Not validating scratch starting at " << rec_start_addr;
+        LOG(debug) << "Not validating scratch starting at " << parsed.start;
         continue;
       }
-      if (rec_checksum == ignored_checksum) {
+      if (parsed.checksum == ignored_checksum) {
         LOG(debug) << "Checksum not computed during recording";
         continue;
-      } else if (rec_checksum == sigbus_checksum) {
-        // This was a SIGBUS equivalent region. During replay, this is either
-        // an explicit SIGBUS region, indicated by the IS_SIGBUS_REGION flag
-        // if the data came from the trace, or an implicit one (which we will
-        // catch below) if the data came from a cloned file.
-        if (m.flags & AddressSpace::Mapping::IS_SIGBUS_REGION) {
-          continue;
-        }
+      } else if (parsed.checksum == sigbus_checksum) {
+        continue;
+      } else {
+        rec_checksum = parsed.checksum;
       }
     } else {
       if (!checksum_segment_filter(m)) {
@@ -383,24 +447,18 @@ static void iterate_checksums(Task* t, ChecksumMode mode,
 
     vector<uint8_t> mem;
     mem.resize(m.map.size());
+    memset(mem.data(), 0, mem.size());
     ssize_t valid_mem_len =
-        t->read_bytes_fallible(m.map.start(), m.map.size(), mem.data());
+        t->read_bytes_fallible(m.map.start(), mem.size(), mem.data());
+    /* Areas not read are treated as zero. We have to do this because
+       mappings not backed by valid file data are not readable during
+       recording but are read as 0 during replay. */
     if (valid_mem_len < 0) {
       /* It is possible for whole mappings to be beyond the extent of the
        * backing file, in which case read_bytes_fallible will return -1.
-       * During replay this will be a SIGBUS region (or, if the file was cloned,
-       * we will end up here again), so skip it now.
        */
       ASSERT(t, valid_mem_len == -1 && errno == EIO);
-      if (VALIDATE_CHECKSUMS == mode) {
-        ASSERT(t, rec_checksum == sigbus_checksum);
-      } else {
-        fprintf(c.checksums_file, "(%x) %s\n", sigbus_checksum,
-                raw_map_line.c_str());
-      }
-      continue;
     }
-    mem.resize(valid_mem_len);
 
     if (m.flags & AddressSpace::Mapping::IS_SYSCALLBUF) {
       /* The syscallbuf consists of a region that's written
@@ -1054,9 +1112,13 @@ static string read_exe_dir() {
   return exe_path;
 }
 
-string exe_directory() {
-  static string exe_path = read_exe_dir();
-  return exe_path;
+string resource_path() {
+  string resource_path = Flags::get().resource_path;
+  if (resource_path.empty()) {
+    static string exe_path = read_exe_dir() + "../";
+    return exe_path;
+  }
+  return resource_path;
 }
 
 /**
@@ -1141,14 +1203,6 @@ bool copy_file(int dest_fd, int src_fd) {
   return true;
 }
 
-void* xmalloc(size_t size) {
-  void* mem_ptr = malloc(size);
-  if (!mem_ptr) {
-    notifying_abort();
-  }
-  return mem_ptr;
-}
-
 bool has_effective_caps(uint64_t caps) {
   struct NativeArch::cap_header header = {.version =
                                               _LINUX_CAPABILITY_VERSION_3,
@@ -1215,11 +1269,6 @@ XSaveLayout xsave_layout_from_trace(const std::vector<CPUIDRecord> records) {
   return layout;
 }
 
-uint64_t rr_signal_mask() {
-  return signal_bit(PerfCounters::TIME_SLICE_SIGNAL) |
-         signal_bit(SYSCALLBUF_DESCHED_SIGNAL);
-}
-
 ScopedFd open_socket(const char* address, unsigned short* port,
                      ProbePort probe) {
   ScopedFd listen_fd(socket(AF_INET, SOCK_STREAM | SOCK_CLOEXEC, 0));
@@ -1398,6 +1447,7 @@ int get_num_cpus() {
 static const uint8_t rdtsc_insn[] = { 0x0f, 0x31 };
 static const uint8_t rdtscp_insn[] = { 0x0f, 0x01, 0xf9 };
 static const uint8_t cpuid_insn[] = { 0x0f, 0xa2 };
+static const uint8_t int3_insn[] = { 0xcc };
 
 TrappedInstruction trapped_instruction_at(Task* t, remote_code_ptr ip) {
   uint8_t insn[sizeof(rdtscp_insn)];
@@ -1415,6 +1465,10 @@ TrappedInstruction trapped_instruction_at(Task* t, remote_code_ptr ip) {
       !memcmp(insn, cpuid_insn, sizeof(cpuid_insn))) {
     return TrappedInstruction::CPUID;
   }
+  if ((size_t)len >= sizeof(int3_insn) &&
+      !memcmp(insn, int3_insn, sizeof(int3_insn))) {
+    return TrappedInstruction::INT3;
+  }
   return TrappedInstruction::NONE;
 }
 
@@ -1425,6 +1479,8 @@ size_t trapped_instruction_len(TrappedInstruction insn) {
     return sizeof(rdtscp_insn);
   } else if (insn == TrappedInstruction::CPUID) {
     return sizeof(cpuid_insn);
+  } else if (insn == TrappedInstruction::INT3) {
+    return sizeof(int3_insn);
   } else {
     return 0;
   }
@@ -1580,6 +1636,23 @@ void write_all(int fd, const void* buf, size_t size) {
   }
 }
 
+ssize_t pwrite_all_fallible(int fd, const void* buf, size_t size, off_t offset) {
+  ssize_t written = 0;
+  while (size > 0) {
+    ssize_t ret = ::pwrite64(fd, buf, size, offset);
+    if (ret <= 0) {
+      if (written > 0) {
+        return written;
+      }
+      return ret;
+    }
+    buf = static_cast<const char*>(buf) + ret;
+    written += ret;
+    size -= ret;
+  }
+  return written;
+}
+
 bool is_directory(const char* path) {
   struct stat buf;
   if (stat(path, &buf) < 0) {
@@ -1632,4 +1705,31 @@ void restore_initial_resource_limits() {
   }
 }
 
+template <typename Arch> static size_t word_size_arch() {
+  return sizeof(typename Arch::signed_long);
+}
+
+size_t word_size(SupportedArch arch) {
+  RR_ARCH_FUNCTION(word_size_arch, arch);
+}
+
+string json_escape(const string& str, size_t pos) {
+  string out;
+  for (size_t i = pos; i < str.size(); ++i) {
+    char c = str[i];
+    if (c < 32) {
+      char buf[8];
+      sprintf(buf, "\\u%04x", c);
+      out += buf;
+    } else if (c == '\\') {
+      out += "\\\\";
+    } else if (c == '\"') {
+      out += "\\\"";
+    } else {
+      out += c;
+    }
+  }
+  return out;
+}
+
 } // namespace rr
